#include <config.h>
#ifndef lint
static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif

#include <stdlib.h>
#include <string.h>

#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYPATCH 20070509

#define YYEMPTY (-1)
#define yyclearin    (yychar = YYEMPTY)
#define yyerrok      (yyerrflag = 0)
#define YYRECOVERING (yyerrflag != 0)

extern int yyparse(void);

static int yygrowstack(void);
#define YYPREFIX "yy"
#line 21 "p.y"
  
/*
 * DESCRIPTION
 *   Simple context-free grammar for parsing the control file. 
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Olivier Beyssac, <ob@r14.freenix.org>
 *  @author Kianusch Sayah Karadji <kianusch.sayah.karadji@sk-tech.net>
 *  @author Martin Pala <martinp@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *  @author Rory Toma <rory@digeo.com>
 *  @version \$Id: p.y,v 1.261 2007/10/17 11:09:55 hauk Exp $
 */

#include <config.h>

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif 

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif 

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif 

#ifdef HAVE_GRP_H
#include <grp.h>
#endif 
  
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
  
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_ASM_PARAM_H
#include <asm/param.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#ifndef HAVE_SOL_IP
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#endif

#ifdef HAVE_NETINET_IP_ICMP_H
#include <netinet/ip_icmp.h>
#endif

#ifdef HAVE_REGEX_H
#include <regex.h>
#endif
  
#include "net.h"
#include "monitor.h"
#include "protocol.h"
#include "engine.h"
#include "alert.h"
#include "process.h"
#include "ssl.h"
#include "device.h"


/* ------------------------------------------------------------- Definitions */

  struct IHavePrecedence {
    int daemon;
    int logfile;
    int pidfile;
  }; 

  struct myrate {
    unsigned count;
    unsigned cycles;
  }; 

  /* yacc interface */
  void  yyerror(const char *,...);
  void  yyerror2(const char *,...);
  void  yywarning(const char *,...);
  void  yywarning2(const char *,...);

  /* lexer interface */
  int yylex(void);
  extern FILE *yyin;
  extern int lineno;
  extern int arglineno;
  extern char *yytext;
  extern char *argyytext;
  extern char *currentfile;
  extern char *argcurrentfile;
  extern int buffer_stack_ptr;

  /* Local variables */
  static int cfg_errflag= FALSE;
  static Service_T tail= NULL;
  static Service_T current= NULL;
  static unsigned int eventset;
  static Request_T urlrequest= NULL;
  static Command_T command= NULL;
  static Command_T command1= NULL;
  static Command_T command2= NULL;
  static Service_T depend_list= NULL;
  static struct mygid gidset;
  static struct myuid uidset;
  static struct myperm permset;
  static struct mysize sizeset;
  static struct mymatch matchset;
  static struct myicmp icmpset;
  static struct mymail mailset;
  static struct myport portset;
  static struct mymailserver mailserverset;
  static struct mydevice deviceset;
  static struct myresource resourceset;
  static struct mychecksum checksumset;
  static struct mytimestamp timestampset;
  static struct IHavePrecedence ihp= {FALSE, FALSE, FALSE};
  static struct myrate rate1 = {1, 1};
  static struct myrate rate2 = {1, 1};
  static char * htpasswd_file= NULL;
  static int    digesttype= DIGEST_CLEARTEXT;
  static int    hassystem = FALSE;

#define BITMAP_MAX (sizeof(long long) * 8)

  
/* -------------------------------------------------------------- Prototypes */

  static void  preparse();
  static void  postparse();
  static void  addservice(Service_T);
  static void  addmail(char *, Mail_T, Mail_T *, unsigned int, unsigned int);
  static void  createservice(int, char *, char *, int (*)(Service_T));
  static void  adddependant(char *);
  static void  addport(Port_T);
  static void  addresource(Resource_T);
  static void  addtimestamp(Timestamp_T, int);
  static void  addsize(Size_T, int);
  static void  adddevice(Device_T);
  static void  addicmp(Icmp_T);
  static void *addprotocol(int);
  static void  addgeneric(Port_T, char*, char*);
  static void  addcommand(int);
  static void  addargument(char *);
  static void  addcollector(URL_T, int, int, char *);
  static void  addmailserver(MailServer_T);
  static int   addcredentials(char *, char *, int, int);
  static void  addhtpasswdentry(char *, char *, int);
  static uid_t get_uid(char *, uid_t);
  static gid_t get_gid(char *, gid_t);
  static void  addchecksum(Checksum_T);
  static void  addperm(Perm_T);
  static void  addmatch(Match_T, int, int);
  static void  addmatchpath(Match_T, int);
  static void  adduid(Uid_T);
  static void  addgid(Gid_T);
  static void  addeuid(uid_t);
  static void  addegid(gid_t);
  static void  addeventaction(EventAction_T *, int, int);
  static void  seteventaction(EventAction_T *, int, int);
  static void  prepare_urlrequest(URL_T U);
  static void  seturlrequest(int, char *);
  static void  setlogfile(char *);
  static void  setpidfile(char *);
  static void  reset_mailset();
  static void  reset_mailserverset();
  static void  reset_portset();
  static void  reset_resourceset();
  static void  reset_timestampset();
  static void  reset_sizeset();
  static void  reset_checksumset();
  static void  reset_permset();
  static void  reset_uidset();
  static void  reset_gidset();
  static void  reset_deviceset();
  static void  reset_icmpset();
  static void  reset_rateset();
  static void  check_name(char *);
  static void  check_timeout(int, int);
  static void  check_every(int);
  static int   check_perm(int);
  static void  check_hostname (char *);
  static void  check_exec(char *);
  static int   cleanup_hash_string(char *);
  static void  check_depend();
  static void  setsyslog(char *);
  static void describeAction(Action_T);
  static Command_T copycommand(Command_T);

#line 241 "p.y"
typedef union {
  URL_T url;
  float real;
  int   number;
  char *string;
} YYSTYPE;
#line 249 "y.tab.c"
#define IF 257
#define ELSE 258
#define THEN 259
#define OR 260
#define FAILED 261
#define SET 262
#define LOGFILE 263
#define FACILITY 264
#define DAEMON 265
#define SYSLOG 266
#define MAILSERVER 267
#define HTTPD 268
#define ALLOW 269
#define ADDRESS 270
#define INIT 271
#define READONLY 272
#define CLEARTEXT 273
#define MD5HASH 274
#define SHA1HASH 275
#define CRYPT 276
#define PEMFILE 277
#define ENABLE 278
#define DISABLE 279
#define HTTPDSSL 280
#define CLIENTPEMFILE 281
#define ALLOWSELFCERTIFICATION 282
#define STATEFILE 283
#define SEND 284
#define EXPECT 285
#define CYCLE 286
#define COUNT 287
#define REMINDER 288
#define PIDFILE 289
#define START 290
#define STOP 291
#define PATHTOK 292
#define HOST 293
#define PORT 294
#define TYPE 295
#define UDP 296
#define TCP 297
#define TCPSSL 298
#define PROTOCOL 299
#define CONNECTION 300
#define ALERT 301
#define NOALERT 302
#define MAILFORMAT 303
#define UNIXSOCKET 304
#define SIGNATURE 305
#define TIMEOUT 306
#define RESTART 307
#define CHECKSUM 308
#define EVERY 309
#define DEFAULT 310
#define HTTP 311
#define APACHESTATUS 312
#define FTP 313
#define SMTP 314
#define POP 315
#define IMAP 316
#define CLAMAV 317
#define NNTP 318
#define NTP3 319
#define MYSQL 320
#define DNS 321
#define SSH 322
#define DWP 323
#define LDAP2 324
#define LDAP3 325
#define RDATE 326
#define RSYNC 327
#define TNS 328
#define PGSQL 329
#define POSTFIXPOLICY 330
#define STRING 331
#define PATH 332
#define MAILADDR 333
#define MAILFROM 334
#define MAILSUBJECT 335
#define MAILBODY 336
#define SERVICENAME 337
#define STRINGNAME 338
#define NUMBER 339
#define PERCENT 340
#define LOGLIMIT 341
#define CLOSELIMIT 342
#define DNSLIMIT 343
#define KEEPALIVELIMIT 344
#define REPLYLIMIT 345
#define REQUESTLIMIT 346
#define STARTLIMIT 347
#define WAITLIMIT 348
#define GRACEFULLIMIT 349
#define CLEANUPLIMIT 350
#define REAL 351
#define CHECKPROC 352
#define CHECKDEV 353
#define CHECKFILE 354
#define CHECKDIR 355
#define CHECKHOST 356
#define CHECKSYSTEM 357
#define CHECKFIFO 358
#define CHILDREN 359
#define SYSTEM 360
#define RESOURCE 361
#define MEMORY 362
#define TOTALMEMORY 363
#define LOADAVG1 364
#define LOADAVG5 365
#define LOADAVG15 366
#define MODE 367
#define ACTIVE 368
#define PASSIVE 369
#define MANUAL 370
#define CPU 371
#define CPUUSER 372
#define CPUSYSTEM 373
#define CPUWAIT 374
#define GROUP 375
#define REQUEST 376
#define DEPENDS 377
#define BASEDIR 378
#define SLOT 379
#define EVENTQUEUE 380
#define UID 381
#define GID 382
#define COLLECTOR 383
#define INSTANCE 384
#define USERNAME 385
#define PASSWORD 386
#define TIMESTAMP 387
#define CHANGED 388
#define SECOND 389
#define MINUTE 390
#define HOUR 391
#define DAY 392
#define SSLAUTO 393
#define SSLV2 394
#define SSLV3 395
#define TLSV1 396
#define CERTMD5 397
#define BYTE 398
#define KILOBYTE 399
#define MEGABYTE 400
#define GIGABYTE 401
#define INODE 402
#define SPACE 403
#define PERMISSION 404
#define SIZE 405
#define MATCH 406
#define NOT 407
#define IGNORE 408
#define EXEC 409
#define UNMONITOR 410
#define ICMP 411
#define ICMPECHO 412
#define NONEXIST 413
#define INVALID 414
#define DATA 415
#define RECOVERED 416
#define PASSED 417
#define URL 418
#define CONTENT 419
#define PID 420
#define PPID 421
#define FSFLAG 422
#define URLOBJECT 423
#define GREATER 424
#define LESS 425
#define EQUAL 426
#define NOTEQUAL 427
#define YYERRCODE 256
short yylhs[] = {                                        -1,
    0,    0,    1,    1,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,   15,   15,   28,   28,   28,   28,   28,   28,
   28,   28,   28,   28,   28,   28,   28,   17,   17,   42,
   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,
   42,   42,   42,   42,   19,   19,   50,   50,   50,   50,
   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,
   21,   21,   54,   54,   54,   54,   54,   54,   54,   54,
   54,   54,   54,   54,   23,   23,   55,   55,   55,   55,
   55,   55,   55,   55,   55,   55,   25,   25,   57,   57,
   57,   57,   57,   57,   27,   27,   59,   59,   59,   59,
   59,   59,   59,   59,   59,   59,   59,   59,    3,    3,
    3,    4,   13,    5,    5,    5,    6,    6,    6,   12,
   11,    7,   64,   64,   65,    8,    9,   10,   69,   69,
   72,   72,   71,   71,   75,   75,   75,   75,   76,   76,
   81,   81,   81,   80,   80,   80,   80,   80,   77,   77,
   85,   85,   86,   86,   78,   82,   83,   83,   84,   84,
   79,   79,   79,   79,   79,   89,   79,   90,   79,   91,
   79,   92,   79,   79,   88,   88,   93,   87,   87,   14,
   14,   16,   18,   20,   22,   24,   26,   29,   29,   30,
   30,   94,   94,   95,   95,   96,   96,   97,   97,   97,
   97,   73,   73,   74,   74,   33,   33,   34,   56,   98,
   98,   99,  106,  100,  100,  100,  100,   68,   68,   67,
   67,   67,   67,   67,  101,  101,  101,  101,  101,  101,
  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,
  101,  101,  101,  101,  101,  101,  101,  111,  111,  112,
  112,  110,  110,  110,  109,  109,  113,  113,  113,  113,
  113,  113,  113,  113,  113,  113,   31,   32,  108,  108,
   66,   66,   35,  105,  105,  115,  115,   36,   36,   36,
   36,   60,  116,   61,   61,  117,  117,  117,  117,  117,
  117,  117,  117,  117,  117,  117,  117,  117,  117,  117,
  117,  117,  117,  117,  117,  117,   62,   62,   70,   70,
  118,  118,  118,   37,   38,   38,   38,   39,   40,  119,
  119,  120,   41,  121,  121,  122,  122,  122,  122,   58,
  127,  127,  128,  128,  128,  123,  129,  130,  130,  130,
  124,  124,  124,  124,  125,  126,  133,  133,  133,  131,
  131,   43,   43,  114,  114,  114,  114,  114,  114,  134,
  134,  134,  134,  134,  135,  135,  135,  135,  135,  135,
  135,  103,  136,  102,  102,  102,  137,  137,  137,  104,
  104,  104,   47,   47,   47,  138,  138,  138,   51,   51,
   52,   52,   53,  132,  132,  132,  132,   44,   49,   49,
   49,   49,  139,  139,   48,   48,   45,   45,   46,   46,
  107,   63,   63,   63,
};
short yylen[] = {                                         2,
    0,    1,    1,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    2,    2,    2,    2,    2,
    2,    2,    0,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    0,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    0,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    0,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    0,    2,    1,    1,
    1,    1,    1,    1,    0,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    7,    4,
    8,    3,    2,    3,    3,    5,    4,    6,    4,    3,
    3,    3,    1,    2,    4,    4,    5,    5,    1,    2,
    5,    7,    0,    2,    1,    1,    1,    1,    1,    4,
    1,    2,    2,    2,    2,    3,    3,    2,    1,    1,
    2,    2,    2,    2,    2,    2,    0,    2,    0,    1,
    5,    2,    3,    3,    3,    0,    4,    0,    5,    0,
    5,    0,    5,    2,    1,    2,    1,    0,    1,    4,
    4,    4,    4,    4,    4,    2,    4,    2,    3,    2,
    3,    1,    2,    1,    2,    1,    1,    2,    2,    2,
    2,    0,    2,    0,    2,   11,   10,   10,   10,    0,
    2,    2,    2,    0,    2,    4,    2,    0,    2,    0,
    1,    1,    1,    1,    0,    3,    2,    2,    2,    2,
    3,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    1,    1,    2,    2,
    2,    0,    2,    4,    1,    3,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    6,    6,    0,    2,
    0,    3,    7,    0,    3,    1,    1,    6,    3,    7,
    1,    2,    2,    1,    2,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    0,    4,    1,    2,
    1,    1,    1,    3,    2,    2,    2,    2,    2,    1,
    2,    1,    6,    1,    2,    1,    1,    1,    1,    6,
    1,    2,    1,    1,    1,    4,    4,    1,    1,    1,
    4,    4,    4,    4,    3,    3,    1,    1,    1,    1,
    1,    9,    6,    0,    1,    1,    1,    1,    1,    0,
    1,    1,    1,    1,    1,    2,    3,    1,    1,    1,
    1,    1,    1,    0,    2,    3,    0,    2,    3,    0,
    6,    6,    8,   10,    7,    0,    1,    1,    8,    9,
    9,    9,    6,    1,    1,    1,    1,    8,    7,    7,
    4,    4,    0,    1,    9,    6,    8,    8,    8,    8,
    2,    0,    2,    3,
};
short yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    3,    5,    6,    7,    8,    9,   10,   11,   12,   13,
   14,   15,   23,   38,   55,   71,    0,   97,  105,    0,
    0,    0,    0,  123,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  196,    0,    4,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   87,   88,   89,   91,   92,   93,   94,   95,
   96,   85,   90,    0,  291,    0,    0,    0,  124,  122,
    0,    0,  139,    0,  130,  131,  292,    0,    0,    0,
    0,    0,  133,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   24,   25,   26,   27,   28,
   29,   30,   31,   32,   33,   34,   35,   36,   37,    0,
    0,   40,   41,   43,   45,   44,   52,   53,   54,   39,
   42,   46,   47,   48,   49,   50,   51,    0,   57,   58,
   59,   61,   60,   65,   66,   67,   62,   63,   64,   56,
   68,   69,   70,    0,   73,   74,   76,   78,   77,   82,
   83,   84,   75,   79,   80,   81,   72,    0,    0,  206,
  207,    0,  202,    0,  293,    0,  325,  326,  327,  328,
  332,    0,  330,   86,    0,    0,    0,    0,   99,  100,
  101,  102,  103,   98,  104,  107,  108,  110,  112,  111,
  116,  117,  118,  109,  113,  114,  115,  106,    0,    0,
    0,    0,    0,  136,  140,  143,  321,  322,  323,    0,
  319,    0,  129,    0,  134,    0,    0,  313,  314,  298,
  310,  315,  297,  308,  312,  301,  303,  311,  296,  307,
  309,  305,  300,  316,  302,  306,  304,  299,    0,  294,
    0,  120,  190,  191,  193,  192,  194,  195,  197,    0,
    0,    0,    0,  357,  358,  359,    0,    0,    0,  334,
  336,  337,  338,  339,    0,    0,    0,    0,    0,  414,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  203,  204,    0,  324,  331,
    0,    0,  289,  348,  349,  350,  344,  343,    0,  341,
  345,    0,  126,    0,  213,    0,    0,    0,    0,  137,
  320,    0,  234,  231,  232,  233,    0,    0,    0,    0,
  295,    0,    0,    0,  369,  365,  366,  367,  368,    0,
    0,    0,    0,    0,    0,    0,    0,  335,    0,  397,
  398,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  221,    0,    0,    0,    0,
    0,    0,  208,  210,  209,  211,  205,    0,    0,    0,
  342,    0,    0,  215,    0,  282,    0,    0,    0,    0,
    0,    0,  144,  145,  146,  147,  148,    0,    0,  159,
  160,  128,    0,  135,  318,    0,    0,  424,  223,    0,
    0,  355,    0,  360,    0,    0,    0,    0,    0,    0,
  385,    0,    0,  361,  356,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  412,  411,
    0,    0,    0,    0,  421,    0,    0,    0,    0,  222,
    0,    0,    0,    0,    0,    0,    0,  141,    0,    0,
    0,    0,    0,  165,  153,  162,  155,  164,  152,  154,
  161,  163,    0,    0,  158,    0,    0,  229,    0,  119,
  227,  225,    0,    0,    0,    0,    0,    0,  258,  352,
  404,  405,  406,  407,  351,  354,  353,  346,    0,    0,
  386,  379,  380,  375,  378,    0,  381,    0,  382,    0,
    0,    0,    0,    0,    0,    0,  371,  372,  373,  374,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  280,    0,  286,  287,    0,    0,    0,    0,
    0,  288,    0,  347,    0,    0,    0,    0,    0,    0,
  156,  157,  166,    0,    0,  121,    0,  260,  261,  237,
    0,    0,  240,  251,  250,  242,  243,  247,  248,  246,
  238,  252,  239,  244,  245,  253,  254,  255,  256,  249,
    0,  259,  277,  278,    0,    0,  333,    0,    0,    0,
    0,    0,    0,    0,    0,  363,  416,    0,    0,    0,
    0,  403,    0,    0,    0,    0,    0,  285,    0,    0,
  283,  290,  340,  142,    0,    0,    0,    0,  187,    0,
  185,  168,  170,  150,  226,    0,  241,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  265,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  395,    0,  410,  409,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  189,  171,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  417,  418,  419,  420,  408,    0,  393,    0,
    0,    0,  399,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  266,
    0,    0,    0,    0,    0,  362,  415,  400,  402,  401,
  219,  217,    0,  264,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  218,  388,    0,    0,    0,  394,
  216,  389,  383,  391,  392,
};
short yydgoto[] = {                                       9,
   10,   11,   12,   13,   14,   15,   16,   17,   18,   19,
   20,   21,   22,   23,   50,   24,   51,   25,   52,   26,
   53,   27,   62,   28,   76,   29,   77,  106,   63,   64,
  109,  110,   65,  112,   66,   67,   68,   69,   70,   71,
  119,  130,  131,  132,  133,  134,  135,  136,  137,  150,
  151,  152,  153,  167,   72,   73,  194,  195,  208,   74,
  249,   97,  252,   92,   93,  214,  327,  404,   82,  220,
  319,   83,  212,  317,  393,  394,  395,  396,  397,  398,
  399,  477,  555,  624,  400,  401,  666,  620,  550,  615,
  616,  617,  621,  172,  295,  173,  297,  291,  371,  411,
  487,  347,  508,  587,  449,  334,  368,  447,  638,  627,
  488,  489,  639,  340,  537,   75,  250,  221,  182,  183,
  269,  270,  271,  272,  273,  274,  309,  310,  311,  312,
  415,  495,  275,  521,  509,  744,  713,  355,  281,
};
short yysindex[] = {                                    385,
 1502, -287, -226, -203, -196, -168, -141, -136,    0,  385,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  -29,    0,    0, -230,
 -256, -128,  -80,    0, -234,  -92,  -82,  156, -300, -138,
 -118, -164,    2,    4,    6,   42,    0,   18,    0,  683,
 -186, 1462, 1511, -236,  -94,  -94,  -41,  -22,  -34,    9,
   21,  -29,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, -116,    0,  -75, 1511,   75,    0,    0,
 -264, -247,    0,   10,    0,    0,    0,   -5,   49,   74,
  116, -138,    0,  284,  321, 1373,  160,  129,  134,  166,
  169,  173,  197,  177, 1153,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -213,
  128,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -204,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, -245,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -238,  226,    0,
    0, -123,    0, -123,    0,  253,    0,    0,    0,    0,
    0,   21,    0,    0,  429, 1373,  160, 1312,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  234,  229,
  239,  187,  236,    0,    0,    0,    0,    0,    0, -115,
    0,  202,    0,  -40,    0,   -5, 1373,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1148,    0,
  244,    0,    0,    0,    0,    0,    0,    0,    0, -267,
 -121, -121, -121,    0,    0,    0, -121, -157,  868,    0,
    0,    0,    0,    0, -121, -242, -121, -251, -121,    0,
  180,  189, -338,  174, -121, -121,  213,  270,  289,  182,
  312,  268, -254,  -63,   12,    0,    0,   12,    0,    0,
 1373, 1189,    0,    0,    0,    0,    0,    0, 1328,    0,
    0, -121,    0,  230,    0,  280,  -40,  225,  224,    0,
    0,  285,    0,    0,    0,    0,  228, -111, 1231,  327,
    0,  340,  299,  338,    0,    0,    0,    0,    0,  295,
 -269, -252,  296,  297,  297, -233,  378,    0, -151,    0,
    0,  -30,  -23,  302,  336,  309,  297,  297,  341,  311,
   68,  101,  297,  313, -149,    0,  243,  364,  237,  318,
  338,  381,    0,    0,    0,    0,    0, 1326,  327,  409,
    0,  330,  187,    0,  228,    0, -222,  339, -108, -101,
  158,  162,    0,    0,    0,    0,    0,    1,  394,    0,
    0,    0,  365,    0,    0,  327,  160,    0,    0,  113,
  -39,    0,  332,    0,  -57,  359,  -57,  360,  442,  443,
    0,  417, -272,    0,    0,  297,  297,  297,  297,  297,
 -244,   38,  451,  452,  297,  -57,  297,  297,    0,    0,
  460,  107,  382,  -57,    0,  384,  116,   25,  116,    0,
  -39,  462,  327,  160, -272,  389,  -40,    0,  392,  398,
  399,  676,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  404,  412,    0,  423,  475,    0,  160,    0,
    0,    0,  -40,  428,  430, 1530,  116,  247,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -272, -272,
    0,    0,    0,    0,    0,  -94,    0,  502,    0,  503,
  505,  508,  509,  511,  441,  515,    0,    0,    0,    0,
  297, -272, -272,  516,  297,  517,  518, -272,  297,  528,
  297,  297,    0,  297,    0,    0,  457,  297,  116,  483,
  160,    0,  502,    0,  228,    0,    0,    0,  459,  461,
    0,    0,    0,  465,  513,    0,  228,    0,    0,    0,
  422, 1157,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  297,    0,    0,    0, -123,  542,    0, -272, -272, -272,
 -272, -272,  297, -272,  541,    0,    0, -272,  543, -272,
 -272,    0,  546, -272,  547,  563,  568,    0,  569,  297,
    0,    0,    0,    0,  461,  461,  461,  558,    0,  461,
    0,    0,    0,    0,    0,  500,    0, -121, -121, -121,
 -121, -121, -121, -121, -121, -121, -121,  577,    0,  579,
   12,  126,  502,  502,  502,  502,  502,  580,  502, -272,
    0, -272,    0,    0, -272,  502, -272, -272, -272, -272,
  582,  461,  461,  461,    0,    0,    0,  536,  506,  507,
  512,  514,  524,  533,  535,  544,  545,  554, 1157, -272,
  557,  557,    0,    0,    0,    0,    0, -272,    0,  502,
  502,  502,    0,  502,  502,  502,  502, -272,  519,  522,
  525,  559,  560,  562,  564,  570,  572,  581,  586,    0,
  502, -221,  583,  589,  502,    0,    0,    0,    0,    0,
    0,    0,  502,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  566, -272, -272,    0,
    0,    0,    0,    0,    0,
};
short yyrindex[] = {                                    855,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  873,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  840,    0,    0,    0,    0,    0,    0,    0,    0,  908,
  949,  987, 1017,    0,    0,    0,    0,    0,    0,    0,
    0, 1026,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   31,    0, 1033, 1047, 1054,    0,    0,
  463, 1063,    0,    0,    0,    0,    0,    0,    0,    0,
    8, 1075,    0,    0,    0,    0, 1089,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  476,
  476,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  591,    0,    0,
    0,  205,    0,  235,    0,    0,    0,    0,    0,    0,
    0,  265,    0,    0,    0,    0,  307,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  523,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1114,    0,   15,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  591,
  588, -110, -110,    0,    0,    0,  588,    0,  638,    0,
    0,    0,    0,    0, -110,  590,  588,  590,  588,    0,
    0,    0,    0,    0,  588, -110,    0,    0,    0,    0,
    0,    0,    0,    0,  337,    0,    0,  396,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  638,    0,
    0,  588,    0,  463,    0,    0,  323,    0, 1130,    0,
    0,    0,    0,    0,    0,    0,   22,    0,    0,  840,
    0,  426,    0,  768,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  638,  638,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  638,  638,    0,    0,
    0,    0,  638,    0,    0,    0,    0, -220, -210,    0,
  768,    0,    0,    0,    0,    0,    0,    0,   31,    0,
    0,    0,  523,    0,  193,    0,    0,    0,    0,    0,
  626,    0,    0,    0,    0,    0,    0,  786,    0,    0,
    0,    0,    0,    0,    0,  840, 1089,    0,    0,    0,
  -73,    0,   70,    0,    0,   70,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  638,  638,  638,  638,  638,
  638, -225,    0,    0,  638,    0,  638,  638,    0,    0,
    0,  638,   70,    0,    0,    0, -212,    0, -212,    0,
  -73,    0,   31,  307,    0,    0,  323,    0,    0,    0,
    0,  807,  645,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  666,    0, 1089,    0,
    0,    0, -239,    0,    0,    0, -212,  -64,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  534,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  638,    0,    0,    0,  638,    0,    0,    0,  638,    0,
  638,  638,    0,  638,    0,    0,    0,  638, -212,    0,
  307,    0,  604,    0,  193,  685,  726,  766,    0,    0,
    0,    0,    0,    0,  826,    0,  809,    0,    0,    0,
  -53,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  638,    0,    0,    0,  133,    0,    0,    0,    0,    0,
    0,    0,  638,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  638,
    0,    0,    0,    0,    0,    0,    0,  847,    0,  867,
    0,    0,    0,    0,    0,    0,    0,  588,  588,  588,
  588,  588,  588,  588,  588,  588,  588,  -52,    0,    0,
  163,    0,  456,  456,  456,  456,  456,    0,  456,    0,
    0,    0,    0,    0,    0,  534,    0,    0,    0,    0,
    0,  966,  978, 1007,    0,    0,    0,  -49,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  663,  663,    0,    0,    0,    0,    0,    0,    0,  456,
  456,  534,    0,  534,  534,  534,  534,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  534,    0,    0,    0,  456,    0,    0,    0,    0,    0,
    0,    0,  534,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,
};
short yygindex[] = {                                      0,
    0,  919,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   40,  366,
    0,    0,  880,    0,  274,  407,  656,  526,  818,  917,
    0,    0,    3,   20,   36,  178,    0,    0,    0,    0,
    0,    0,    0,    0,  869,    0,    0,    0,    0,  931,
 -169,  -61, -159,    0,  841,  -90, -305, -374,    0,  708,
    0,  856,  623,  556,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -104,    0,    0,
    0,    0, -349,  -56, -171, -170, -289,    0,    0,  571,
  490, -282, 1068, -479,    0,    0,    0,    0,    0,    0,
    0,  455,  272, -155,    0,    0, -228, -152,    0,  770,
    0,  684,    0, -150,    0, -148,    0,  668,    0,    0,
  -68, -375,    0,    0, -185,  233,  293,  702,  861,
};
#define YYTABLESIZE 1888
short yytable[] = {                                     174,
  224,  296,  298,  296,   96,  377,  186,  281,  377,  320,
  458,  385,  187,  405,  230,  283,  302,  502,  503,  230,
  331,  228,  350,  351,  168,  288,  380,  303,  504,  210,
  317,  350,  351,  370,  505,   78,  333,  307,  279,  308,
  515,  497,  352,  353,  230,  230,  281,  276,  284,   42,
  459,  460,  421,  461,  288,  163,  283,  329,  213,  230,
  525,  419,  420,  613,  736,  354,  230,  321,  532,  413,
  120,  147,  164,  331,  433,  434,  373,   89,   90,  204,
  441,  414,   80,   81,  374,  279,  416,  148,  165,  107,
  122,  139,  155,  169,  346,  284,  205,   85,  414,  230,
  331,   79,  169,   55,   56,  422,  341,  342,  462,  463,
   43,  343,  206,  370,   37,   57,  196,  737,  279,  349,
  211,  356,   58,  360,   98,  169,  281,   99,  284,  364,
  365,  378,  376,   44,  169,  357,  506,  507,  352,  353,
   45,  277,  287,  510,  511,  512,  513,  514,  516,  331,
  290,  545,  524,  358,  526,  527,  382,  230,  307,  530,
  308,  354,  377,  683,  684,  685,  686,  687,   46,  689,
  614,  465,  289,  277,  278,  321,  693,  557,  467,  290,
   59,  188,  625,  284,   94,  235,   94,  424,   60,  443,
   61,  279,  228,  280,  257,   47,  466,  285,  286,  414,
   48,  414,   81,  468,  198,  262,  236,  170,  171,  263,
  716,  717,  718,   84,  719,  720,  721,  722,  217,  218,
  219,  121,  217,  218,  219,   37,   57,   54,  364,  149,
  166,  735,  235,   58,  200,  740,  170,  171,  595,   86,
  364,  257,  599,  741,  484,  485,  603,  480,  605,  606,
   87,  607,  262,  236,  207,  609,  263,  293,  294,  486,
   55,   56,  344,  345,  329,  235,  335,  375,  407,  281,
  667,   37,   57,  417,  257,  376,  230,  473,   88,   58,
  425,  474,  475,  228,   91,  262,  236,  317,   95,  263,
  185,  175,  317,  100,  542,  101,  444,  102,  640,   60,
  426,   61,  336,  337,  338,  339,  422,  428,  427,  104,
  648,  103,  667,  667,  667,  429,  176,  454,  317,  556,
  317,  317,  230,  113,  124,  141,  157,  661,  217,  218,
  219,  317,  317,  177,  178,  179,  199,   59,  209,  317,
  491,  492,  493,  494,  479,   60,  180,   61,  216,  189,
  198,  377,  323,  324,  325,  326,  534,  181,  538,  281,
  281,  281,  281,  281,  281,  281,  230,  230,  230,  230,
  230,  230,  230,  228,  228,  228,  228,  228,  228,  228,
  222,  612,  317,  317,  317,  317,  317,  317,  317,  376,
  376,  541,  293,  294,  376,  201,  581,  317,  437,  438,
  281,  281,  281,  281,  281,  317,  226,  317,  481,  482,
  483,  230,  223,  641,  296,  108,  123,  140,  156,  377,
  377,  213,  376,  376,  377,  423,  517,  518,  519,  520,
  281,  439,  440,  376,  376,  469,  470,  230,  317,  471,
  472,  376,  197,  227,  228,  346,  529,  251,  610,  585,
  535,  536,  377,  377,  228,  390,  114,  125,  142,  158,
  253,  198,  212,  377,  377,  254,  198,  361,  361,  361,
  361,  377,  669,  670,  671,  672,  673,  674,  675,  676,
  677,  678,  190,  199,  376,  376,  376,  376,  376,  376,
  376,  200,  387,  388,  198,  198,  200,  255,  228,  376,
  256,  389,  390,  391,  257,  198,  198,  376,  259,  376,
  662,  663,  664,  198,  377,  377,  377,  377,  377,  377,
  377,  329,  214,  228,  200,  200,  329,  258,  392,  377,
  484,  485,  292,  390,  280,  200,  200,  377,  299,  377,
  376,  681,  682,  200,  228,  228,  228,  228,  228,  228,
  228,  301,  743,  743,  329,  329,  198,  198,  198,  198,
  198,  198,  198,  422,  313,  329,  329,  314,  422,  315,
  377,  198,  316,  329,  318,  116,  127,  144,  160,  198,
  322,  198,  332,  367,  230,  361,  200,  200,  200,  200,
  200,  200,  200,  199,  362,  363,  422,  422,  199,  357,
  366,  200,  201,  390,  369,  370,  372,  422,  422,  200,
  384,  200,  198,  386,  211,  422,  329,  329,  329,  329,
  329,  329,  329,  402,  403,  408,  199,  199,  230,   94,
  409,  329,  410,  412,  418,  346,  423,  199,  199,  329,
  430,  329,  200,  431,  172,  199,    1,  432,  435,  436,
  446,  442,  201,  230,  445,  448,  450,  201,  422,  422,
  422,  422,  422,  422,  422,  167,  452,  455,  456,  464,
  476,  490,  329,  422,  230,  230,  230,  230,  230,  230,
  230,  422,  423,  422,  173,  201,  201,  423,  199,  199,
  199,  199,  199,  199,  199,  478,  201,  201,  496,  498,
  499,  500,  501,  199,  201,  115,  126,  143,  159,  522,
  523,  199,  390,  199,  422,  423,  423,  390,  528,  230,
  540,  531,  533,  546,  212,  174,  423,  423,  544,  547,
  548,  191,  200,  549,  423,  551,    2,    3,    4,    5,
    6,    7,    8,  552,  199,  390,  390,  201,  201,  201,
  201,  201,  201,  201,  553,  554,  390,  390,  558,  586,
  559,  588,  201,  589,  390,  175,  590,  591,  212,  592,
  201,  593,  201,  594,  598,  600,  601,  423,  423,  423,
  423,  423,  423,  423,  214,  149,  604,  608,  611,  618,
  390,  619,  423,  212,  623,  390,  622,  626,  642,  650,
  423,  652,  423,  201,  655,  657,  184,  390,  390,  390,
  390,  390,  390,  390,  212,  212,  212,  212,  212,  212,
  212,  658,  390,  390,  390,  169,  659,  660,  214,  665,
  390,  668,  390,  423,  390,  390,  679,  680,  688,  317,
  698,  738,  390,  699,  700,  701,  188,  739,  212,  724,
  702,  742,  703,  214,    1,  212,  212,  212,  212,  212,
  390,  725,  704,  390,  726,  390,  177,  117,  128,  145,
  161,  705,    2,  706,  214,  214,  214,  214,  214,  214,
  214,  413,  707,  708,  220,  390,  390,  390,  390,  390,
  390,  390,  709,  192,  202,  712,  384,  396,  727,  728,
  390,  729,  151,  730,  390,  390,  172,   16,  390,  731,
  390,  732,  390,  172,  172,  214,  214,  214,  214,  214,
  733,  387,  172,  172,  172,  734,  364,  167,   49,  111,
  184,   41,  225,  328,  167,  167,  383,  215,  457,  105,
  539,  451,  582,  167,  167,  167,  173,  167,   17,  172,
  710,  300,  348,  173,  173,  390,  390,  390,  390,  390,
  390,  390,  173,  173,  173,  179,  118,  129,  146,  162,
  167,  745,   55,   56,  714,  176,  381,  181,  390,  359,
  390,  282,    0,   37,   57,    0,   18,  174,    0,  173,
    0,   58,  193,  203,  174,  174,  172,  172,  172,  172,
  172,  172,  172,  174,  174,  174,  183,    0,    0,    0,
    0,    0,    0,    0,    0,  178,   19,  167,  167,  167,
  167,  167,  167,  167,    0,   20,  224,  175,    0,    0,
  174,    0,   21,    0,  175,  175,  173,  173,  173,  173,
  173,  173,  173,  175,  175,  175,   22,  149,    0,   59,
    0,  224,  224,  125,  149,  149,  180,   60,    0,   61,
    0,    0,  281,  149,  149,  149,  224,  228,  184,    0,
  175,    0,    0,  224,  132,  184,  184,  174,  174,  174,
  174,  174,  174,  174,  184,  184,  184,  169,  422,    0,
  149,    0,  228,  228,  169,  169,  182,    0,    0,    0,
    0,  317,    0,  169,  169,  169,  224,  228,  188,    0,
    0,  184,    0,  127,  228,  188,  188,  175,  175,  175,
  175,  175,  175,  175,  188,  188,  188,  317,  177,  138,
  169,    0,    0,    0,    0,  177,  177,  149,  149,  149,
  149,  149,  149,  149,  177,  177,  177,  228,    0,    0,
    0,  188,    0,    0,    0,    0,    0,    0,  184,  184,
  184,  184,  184,  184,  184,    0,    0,    0,    0,   16,
    0,  177,    0,    0,    0,    0,    0,  169,  169,  169,
  169,  169,  169,  169,    0,    0,    0,    0,    0,    0,
    0,  317,  317,  317,  317,  317,  317,  317,  188,  188,
  188,  188,  188,  188,  188,    0,  346,    0,    0,    0,
   17,    0,    0,    0,    0,    0,    0,    0,  177,  177,
  177,  177,  177,  177,  177,    0,  261,  179,    0,  262,
  263,  264,  265,  266,  179,  179,    0,    0,  267,  181,
    0,    0,    0,  179,  179,  179,  181,  181,   18,    0,
    0,    0,    0,    0,    0,  181,  181,  181,    0,   16,
   16,   16,   16,   16,   16,   16,    0,    0,  183,    0,
  179,    0,  330,    0,    0,  183,  183,    0,   19,    0,
    0,    0,  181,    0,  183,  183,  183,   20,    0,    0,
    0,    0,    0,    0,   21,    0,    0,    0,    0,    0,
   17,   17,   17,   17,   17,   17,   17,    0,   22,    0,
    0,  183,    0,  379,    0,  125,    0,  179,  179,  179,
  179,  179,  179,  179,  281,    0,    0,    0,    0,  181,
  181,  181,  181,  181,  181,  181,  132,    0,   18,   18,
   18,   18,   18,   18,   18,    0,    0,    0,    0,    0,
  422,    0,    0,    0,    0,  406,    0,    0,  183,  183,
  183,  183,  183,  183,  183,    0,    0,    0,   19,   19,
   19,   19,   19,   19,   19,  127,    0,   20,   20,   20,
   20,   20,   20,   20,   21,   21,   21,   21,   21,   21,
   21,  138,    0,    0,    0,    0,    0,    0,   22,   22,
   22,   22,   22,   22,   22,  125,  125,  125,  125,  125,
  125,  125,    0,  260,  281,  281,  281,  281,  281,  281,
  281,    0,    0,    0,    0,    0,  132,  132,  132,  132,
  132,  132,  132,    0,    0,    0,    0,  228,  229,    0,
  422,  422,  422,  422,  422,  422,  422,  230,    0,    0,
  453,    0,    0,  231,  232,  233,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  127,  127,  127,  127,  127,
  127,  127,    0,    0,    0,    0,    0,    0,  228,  229,
    0,  138,  138,  138,  138,  138,  138,  138,  230,    0,
    0,  169,    0,    0,  231,  232,  233,  628,  629,  630,
  631,  632,  633,  634,  635,  636,  637,    0,  234,    0,
    0,  261,    0,    0,  262,  263,  264,  265,  266,    0,
  228,  229,  543,  267,    0,    0,    0,    0,  235,  236,
  230,  237,    0,    0,  238,  239,  231,  232,  233,    0,
  268,    0,    0,    0,    0,    0,    0,    0,    0,  234,
    0,  240,  241,  242,    0,    0,  243,  244,  245,    0,
  246,  247,  248,    0,    0,    0,  583,  584,    0,  235,
  236,    0,  237,    0,    0,  238,  239,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  596,
  597,  234,  240,  241,  242,  602,    0,  243,  244,  245,
    0,  246,  247,  248,    0,    0,    0,    0,    0,    0,
    0,  235,  236,    0,  237,  228,  229,  238,  239,    0,
    0,    0,    0,    0,    0,  230,    0,    0,    0,    0,
    0,  231,  232,  233,  240,  241,  242,    0,    0,  243,
  244,  245,    0,  246,  247,  248,    0,    0,    0,    0,
  169,    0,    0,    0,    0,  643,  644,  645,  646,  647,
    0,  649,  228,  229,    0,  651,  346,  653,  654,    0,
    0,  656,  230,  262,  263,  264,  265,  266,  231,  232,
  233,    0,    0,  304,  305,  306,  234,    0,    0,  262,
  263,  264,  265,  266,    0,    0,    0,    0,    0,  304,
  305,  306,    0,    0,    0,    0,  235,  236,    0,  237,
    0,    0,  238,  239,    0,    0,    0,  690,  138,  691,
    0,    0,  692,    0,  694,  695,  696,  697,    0,  240,
  241,  242,    0,  234,  243,  244,  245,    0,  246,  247,
  248,    0,    0,    0,    0,    0,    0,  711,    0,    0,
    0,   55,   56,  235,  236,  715,  237,    0,    0,  238,
  239,    0,   37,   57,   30,  723,   31,  154,   32,   33,
   58,    0,   34,    0,    0,    0,  240,  241,  242,    0,
    0,  243,  244,  245,   35,  246,  247,  248,    0,    0,
   36,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   55,   56,   37,    0,   38,    0,    0,    0,    0,    0,
    0,   37,   57,    0,    0,    0,    0,    0,    0,   58,
    0,    0,    0,    0,    0,    0,    0,    0,   59,    0,
    0,    0,    0,    0,    0,    0,   60,    0,   61,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,  577,  578,  579,  580,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   59,    0,    0,
    0,   39,    0,    0,   40,   60,    0,   61,
};
short yycheck[] = {                                      56,
   91,  172,  174,  174,  123,  295,  123,    0,  298,  125,
  385,  317,   74,  125,    0,  261,  186,  290,  291,  259,
  249,    0,  274,  275,  261,  293,  309,  187,  301,  294,
    0,  274,  275,  259,  307,  266,  304,  188,  259,  188,
  285,  417,  381,  382,  284,  285,  259,  261,  259,  337,
  273,  274,  286,  276,  293,   53,  261,  227,  306,  299,
  436,  344,  345,  543,  286,  404,  306,  220,  444,  339,
  257,   52,   53,  302,  357,  358,  331,  378,  379,   77,
  363,  351,  339,  331,  339,  306,  339,   52,   53,   50,
   51,   52,   53,  339,  339,  306,   77,  332,  351,  339,
  329,  332,  339,  290,  291,  339,  262,  263,  331,  332,
  337,  267,   77,  339,  301,  302,   77,  339,  339,  275,
  385,  277,  309,  279,  289,  339,  339,  292,  339,  285,
  286,  301,    0,  337,  339,  387,  409,  410,  381,  382,
  337,  387,  388,  426,  427,  428,  429,  430,  431,  378,
  418,  457,  435,  405,  437,  438,  312,  397,  309,  442,
  309,  404,    0,  643,  644,  645,  646,  647,  337,  649,
  545,  280,  411,  387,  388,  328,  656,  483,  280,  418,
  367,  257,  557,  388,  303,  259,  303,  339,  375,  339,
  377,  405,    0,  407,  259,  337,  305,  402,  403,  351,
  337,  351,  331,  305,    0,  259,  259,  331,  332,  259,
  690,  691,  692,  294,  694,  695,  696,  697,  334,  335,
  336,  408,  334,  335,  336,  301,  302,  257,  339,   52,
   53,  711,  306,  309,    0,  715,  331,  332,  521,  332,
  351,  306,  525,  723,  284,  285,  529,  407,  531,  532,
  333,  534,  306,  306,   77,  538,  306,  381,  382,  299,
  290,  291,  420,  421,    0,  339,  388,  331,  330,  262,
  620,  301,  302,  342,  339,  339,  262,  277,  123,  309,
  349,  281,  282,  262,  423,  339,  339,  257,  407,  339,
  407,  333,  262,  292,  454,  292,  365,  292,  581,  375,
  331,  377,  424,  425,  426,  427,    0,  331,  339,  292,
  593,  270,  662,  663,  664,  339,  339,  379,  288,  479,
  290,  291,    0,   50,   51,   52,   53,  610,  334,  335,
  336,  301,  302,  368,  369,  370,    0,  367,  264,  309,
  398,  399,  400,  401,  406,  375,  338,  377,  339,   76,
   77,  641,  393,  394,  395,  396,  447,  337,  449,  352,
  353,  354,  355,  356,  357,  358,  352,  353,  354,  355,
  356,  357,  358,  352,  353,  354,  355,  356,  357,  358,
  332,  541,  352,  353,  354,  355,  356,  357,  358,  257,
  258,  453,  381,  382,  262,    0,  487,  367,  331,  332,
  393,  394,  395,  396,  397,  375,  123,  377,  296,  297,
  298,  397,  339,  585,  585,   50,   51,   52,   53,  257,
  258,  306,  290,  291,  262,    0,  389,  390,  391,  392,
  423,  331,  332,  301,  302,  278,  279,  423,  408,  278,
  279,  309,   77,  123,  423,  339,  340,  288,  539,  506,
  426,  427,  290,  291,  262,    0,   50,   51,   52,   53,
  332,  257,    0,  301,  302,  332,  262,  398,  399,  400,
  401,  309,  628,  629,  630,  631,  632,  633,  634,  635,
  636,  637,   76,   77,  352,  353,  354,  355,  356,  357,
  358,  257,  269,  270,  290,  291,  262,  332,  306,  367,
  332,  278,  279,  280,  332,  301,  302,  375,  332,  377,
  615,  616,  617,  309,  352,  353,  354,  355,  356,  357,
  358,  257,    0,  331,  290,  291,  262,  331,  305,  367,
  284,  285,  307,    0,  407,  301,  302,  375,  286,  377,
  408,  416,  417,  309,  352,  353,  354,  355,  356,  357,
  358,  123,  738,  739,  290,  291,  352,  353,  354,  355,
  356,  357,  358,  257,  331,  301,  302,  339,  262,  331,
  408,  367,  386,  309,  339,   50,   51,   52,   53,  375,
  379,  377,  339,  295,  262,  406,  352,  353,  354,  355,
  356,  357,  358,  257,  406,  422,  290,  291,  262,  387,
  331,  367,   77,    0,  423,  294,  339,  301,  302,  375,
  331,  377,  408,  389,  385,  309,  352,  353,  354,  355,
  356,  357,  358,  339,  397,  286,  290,  291,  306,  303,
  332,  367,  295,  339,  339,  339,  259,  301,  302,  375,
  339,  377,  408,  308,    0,  309,  262,  339,  308,  339,
  287,  339,  257,  331,  412,  419,  339,  262,  352,  353,
  354,  355,  356,  357,  358,    0,  286,  259,  339,  331,
  277,  340,  408,  367,  352,  353,  354,  355,  356,  357,
  358,  375,  257,  377,    0,  290,  291,  262,  352,  353,
  354,  355,  356,  357,  358,  331,  301,  302,  340,  340,
  259,  259,  286,  367,  309,   50,   51,   52,   53,  259,
  259,  375,  257,  377,  408,  290,  291,  262,  259,  397,
  259,  340,  339,  332,  262,    0,  301,  302,  340,  332,
  332,   76,   77,   58,  309,  332,  352,  353,  354,  355,
  356,  357,  358,  332,  408,  290,  291,  352,  353,  354,
  355,  356,  357,  358,  332,  281,  301,  302,  331,  258,
  331,  259,  367,  259,  309,    0,  259,  259,  306,  259,
  375,  331,  377,  259,  259,  259,  259,  352,  353,  354,
  355,  356,  357,  358,  262,    0,  259,  331,  306,  331,
  257,  331,  367,  331,  282,  262,  332,  376,  257,  259,
  375,  259,  377,  408,  259,  259,    0,  352,  353,  354,
  355,  356,  357,  358,  352,  353,  354,  355,  356,  357,
  358,  259,  367,  290,  291,    0,  259,  259,  306,  272,
  375,  332,  377,  408,  301,  302,  260,  259,  259,    0,
  259,  259,  309,  308,  339,  339,    0,  259,  386,  331,
  339,  286,  339,  331,    0,  393,  394,  395,  396,  397,
  257,  340,  339,  408,  340,  262,    0,   50,   51,   52,
   53,  339,    0,  339,  352,  353,  354,  355,  356,  357,
  358,  406,  339,  339,  294,  352,  353,  354,  355,  356,
  357,  358,  339,   76,   77,  339,  259,  308,  340,  340,
  367,  340,  277,  340,  301,  302,  262,    0,  375,  340,
  377,  340,  309,  269,  270,  393,  394,  395,  396,  397,
  340,  259,  278,  279,  280,  340,  339,  262,   10,   50,
   62,    1,   92,  226,  269,  270,  314,   82,  383,  257,
  451,  371,  488,  278,  279,  280,  262,  282,    0,  305,
  679,  182,  269,  269,  270,  352,  353,  354,  355,  356,
  357,  358,  278,  279,  280,    0,   50,   51,   52,   53,
  305,  739,  290,  291,  682,  331,  309,    0,  375,  278,
  377,  121,   -1,  301,  302,   -1,    0,  262,   -1,  305,
   -1,  309,   76,   77,  269,  270,  352,  353,  354,  355,
  356,  357,  358,  278,  279,  280,    0,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  331,    0,  352,  353,  354,
  355,  356,  357,  358,   -1,    0,  259,  262,   -1,   -1,
  305,   -1,    0,   -1,  269,  270,  352,  353,  354,  355,
  356,  357,  358,  278,  279,  280,    0,  262,   -1,  367,
   -1,  284,  285,    0,  269,  270,  331,  375,   -1,  377,
   -1,   -1,    0,  278,  279,  280,  299,  259,  262,   -1,
  305,   -1,   -1,  306,    0,  269,  270,  352,  353,  354,
  355,  356,  357,  358,  278,  279,  280,  262,    0,   -1,
  305,   -1,  284,  285,  269,  270,  331,   -1,   -1,   -1,
   -1,  262,   -1,  278,  279,  280,  339,  299,  262,   -1,
   -1,  305,   -1,    0,  306,  269,  270,  352,  353,  354,
  355,  356,  357,  358,  278,  279,  280,  288,  262,    0,
  305,   -1,   -1,   -1,   -1,  269,  270,  352,  353,  354,
  355,  356,  357,  358,  278,  279,  280,  339,   -1,   -1,
   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,  352,  353,
  354,  355,  356,  357,  358,   -1,   -1,   -1,   -1,  262,
   -1,  305,   -1,   -1,   -1,   -1,   -1,  352,  353,  354,
  355,  356,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  352,  353,  354,  355,  356,  357,  358,  352,  353,
  354,  355,  356,  357,  358,   -1,  339,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  352,  353,
  354,  355,  356,  357,  358,   -1,  359,  262,   -1,  362,
  363,  364,  365,  366,  269,  270,   -1,   -1,  371,  262,
   -1,   -1,   -1,  278,  279,  280,  269,  270,  262,   -1,
   -1,   -1,   -1,   -1,   -1,  278,  279,  280,   -1,  352,
  353,  354,  355,  356,  357,  358,   -1,   -1,  262,   -1,
  305,   -1,  125,   -1,   -1,  269,  270,   -1,  262,   -1,
   -1,   -1,  305,   -1,  278,  279,  280,  262,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
  352,  353,  354,  355,  356,  357,  358,   -1,  262,   -1,
   -1,  305,   -1,  125,   -1,  262,   -1,  352,  353,  354,
  355,  356,  357,  358,  262,   -1,   -1,   -1,   -1,  352,
  353,  354,  355,  356,  357,  358,  262,   -1,  352,  353,
  354,  355,  356,  357,  358,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,  125,   -1,   -1,  352,  353,
  354,  355,  356,  357,  358,   -1,   -1,   -1,  352,  353,
  354,  355,  356,  357,  358,  262,   -1,  352,  353,  354,
  355,  356,  357,  358,  352,  353,  354,  355,  356,  357,
  358,  262,   -1,   -1,   -1,   -1,   -1,   -1,  352,  353,
  354,  355,  356,  357,  358,  352,  353,  354,  355,  356,
  357,  358,   -1,  261,  352,  353,  354,  355,  356,  357,
  358,   -1,   -1,   -1,   -1,   -1,  352,  353,  354,  355,
  356,  357,  358,   -1,   -1,   -1,   -1,  290,  291,   -1,
  352,  353,  354,  355,  356,  357,  358,  300,   -1,   -1,
  125,   -1,   -1,  306,  307,  308,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  352,  353,  354,  355,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,
   -1,  352,  353,  354,  355,  356,  357,  358,  300,   -1,
   -1,  339,   -1,   -1,  306,  307,  308,  341,  342,  343,
  344,  345,  346,  347,  348,  349,  350,   -1,  361,   -1,
   -1,  359,   -1,   -1,  362,  363,  364,  365,  366,   -1,
  290,  291,  455,  371,   -1,   -1,   -1,   -1,  381,  382,
  300,  384,   -1,   -1,  387,  388,  306,  307,  308,   -1,
  388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  361,
   -1,  404,  405,  406,   -1,   -1,  409,  410,  411,   -1,
  413,  414,  415,   -1,   -1,   -1,  499,  500,   -1,  381,
  382,   -1,  384,   -1,   -1,  387,  388,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  522,
  523,  361,  404,  405,  406,  528,   -1,  409,  410,  411,
   -1,  413,  414,  415,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  381,  382,   -1,  384,  290,  291,  387,  388,   -1,
   -1,   -1,   -1,   -1,   -1,  300,   -1,   -1,   -1,   -1,
   -1,  306,  307,  308,  404,  405,  406,   -1,   -1,  409,
  410,  411,   -1,  413,  414,  415,   -1,   -1,   -1,   -1,
  339,   -1,   -1,   -1,   -1,  588,  589,  590,  591,  592,
   -1,  594,  290,  291,   -1,  598,  339,  600,  601,   -1,
   -1,  604,  300,  362,  363,  364,  365,  366,  306,  307,
  308,   -1,   -1,  372,  373,  374,  361,   -1,   -1,  362,
  363,  364,  365,  366,   -1,   -1,   -1,   -1,   -1,  372,
  373,  374,   -1,   -1,   -1,   -1,  381,  382,   -1,  384,
   -1,   -1,  387,  388,   -1,   -1,   -1,  650,  257,  652,
   -1,   -1,  655,   -1,  657,  658,  659,  660,   -1,  404,
  405,  406,   -1,  361,  409,  410,  411,   -1,  413,  414,
  415,   -1,   -1,   -1,   -1,   -1,   -1,  680,   -1,   -1,
   -1,  290,  291,  381,  382,  688,  384,   -1,   -1,  387,
  388,   -1,  301,  302,  263,  698,  265,  257,  267,  268,
  309,   -1,  271,   -1,   -1,   -1,  404,  405,  406,   -1,
   -1,  409,  410,  411,  283,  413,  414,  415,   -1,   -1,
  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,  301,   -1,  303,   -1,   -1,   -1,   -1,   -1,
   -1,  301,  302,   -1,   -1,   -1,   -1,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  367,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1,  377,  310,
  311,  312,  313,  314,  315,  316,  317,  318,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  367,   -1,   -1,
   -1,  380,   -1,   -1,  383,  375,   -1,  377,
};
#define YYFINAL 9
#ifndef YYDEBUG
#define YYDEBUG 1
#endif
#define YYMAXTOKEN 427
#if YYDEBUG
char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"':'",0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"IF","ELSE","THEN",
"OR","FAILED","SET","LOGFILE","FACILITY","DAEMON","SYSLOG","MAILSERVER","HTTPD",
"ALLOW","ADDRESS","INIT","READONLY","CLEARTEXT","MD5HASH","SHA1HASH","CRYPT",
"PEMFILE","ENABLE","DISABLE","HTTPDSSL","CLIENTPEMFILE",
"ALLOWSELFCERTIFICATION","STATEFILE","SEND","EXPECT","CYCLE","COUNT","REMINDER",
"PIDFILE","START","STOP","PATHTOK","HOST","PORT","TYPE","UDP","TCP","TCPSSL",
"PROTOCOL","CONNECTION","ALERT","NOALERT","MAILFORMAT","UNIXSOCKET","SIGNATURE",
"TIMEOUT","RESTART","CHECKSUM","EVERY","DEFAULT","HTTP","APACHESTATUS","FTP",
"SMTP","POP","IMAP","CLAMAV","NNTP","NTP3","MYSQL","DNS","SSH","DWP","LDAP2",
"LDAP3","RDATE","RSYNC","TNS","PGSQL","POSTFIXPOLICY","STRING","PATH",
"MAILADDR","MAILFROM","MAILSUBJECT","MAILBODY","SERVICENAME","STRINGNAME",
"NUMBER","PERCENT","LOGLIMIT","CLOSELIMIT","DNSLIMIT","KEEPALIVELIMIT",
"REPLYLIMIT","REQUESTLIMIT","STARTLIMIT","WAITLIMIT","GRACEFULLIMIT",
"CLEANUPLIMIT","REAL","CHECKPROC","CHECKDEV","CHECKFILE","CHECKDIR","CHECKHOST",
"CHECKSYSTEM","CHECKFIFO","CHILDREN","SYSTEM","RESOURCE","MEMORY","TOTALMEMORY",
"LOADAVG1","LOADAVG5","LOADAVG15","MODE","ACTIVE","PASSIVE","MANUAL","CPU",
"CPUUSER","CPUSYSTEM","CPUWAIT","GROUP","REQUEST","DEPENDS","BASEDIR","SLOT",
"EVENTQUEUE","UID","GID","COLLECTOR","INSTANCE","USERNAME","PASSWORD",
"TIMESTAMP","CHANGED","SECOND","MINUTE","HOUR","DAY","SSLAUTO","SSLV2","SSLV3",
"TLSV1","CERTMD5","BYTE","KILOBYTE","MEGABYTE","GIGABYTE","INODE","SPACE",
"PERMISSION","SIZE","MATCH","NOT","IGNORE","EXEC","UNMONITOR","ICMP","ICMPECHO",
"NONEXIST","INVALID","DATA","RECOVERED","PASSED","URL","CONTENT","PID","PPID",
"FSFLAG","URLOBJECT","GREATER","LESS","EQUAL","NOTEQUAL",
};
char *yyrule[] = {
"$accept : cfgfile",
"cfgfile :",
"cfgfile : statement_list",
"statement_list : statement",
"statement_list : statement_list statement",
"statement : setalert",
"statement : setdaemon",
"statement : setlog",
"statement : seteventqueue",
"statement : setcollectors",
"statement : setmailservers",
"statement : setmailformat",
"statement : sethttpd",
"statement : setpid",
"statement : setstate",
"statement : setinit",
"statement : checkproc optproclist",
"statement : checkfile optfilelist",
"statement : checkdev optdevlist",
"statement : checkdir optdirlist",
"statement : checkhost opthostlist",
"statement : checksystem optsystemlist",
"statement : checkfifo optfifolist",
"optproclist :",
"optproclist : optproclist optproc",
"optproc : start",
"optproc : stop",
"optproc : pid",
"optproc : ppid",
"optproc : connection",
"optproc : connectionunix",
"optproc : timeout",
"optproc : alert",
"optproc : every",
"optproc : mode",
"optproc : group",
"optproc : depend",
"optproc : resourceprocess",
"optfilelist :",
"optfilelist : optfilelist optfile",
"optfile : start",
"optfile : stop",
"optfile : timestamp",
"optfile : timeout",
"optfile : every",
"optfile : alert",
"optfile : permission",
"optfile : uid",
"optfile : gid",
"optfile : checksum",
"optfile : size",
"optfile : match",
"optfile : mode",
"optfile : group",
"optfile : depend",
"optdevlist :",
"optdevlist : optdevlist optdev",
"optdev : start",
"optdev : stop",
"optdev : timeout",
"optdev : every",
"optdev : alert",
"optdev : permission",
"optdev : uid",
"optdev : gid",
"optdev : mode",
"optdev : group",
"optdev : depend",
"optdev : inode",
"optdev : space",
"optdev : fsflag",
"optdirlist :",
"optdirlist : optdirlist optdir",
"optdir : start",
"optdir : stop",
"optdir : timestamp",
"optdir : timeout",
"optdir : every",
"optdir : alert",
"optdir : permission",
"optdir : uid",
"optdir : gid",
"optdir : mode",
"optdir : group",
"optdir : depend",
"opthostlist : opthost",
"opthostlist : opthostlist opthost",
"opthost : start",
"opthost : stop",
"opthost : connection",
"opthost : icmp",
"opthost : timeout",
"opthost : alert",
"opthost : every",
"opthost : mode",
"opthost : group",
"opthost : depend",
"optsystemlist :",
"optsystemlist : optsystemlist optsystem",
"optsystem : timeout",
"optsystem : alert",
"optsystem : every",
"optsystem : group",
"optsystem : depend",
"optsystem : resourcesystem",
"optfifolist :",
"optfifolist : optfifolist optfifo",
"optfifo : start",
"optfifo : stop",
"optfifo : timestamp",
"optfifo : timeout",
"optfifo : every",
"optfifo : alert",
"optfifo : permission",
"optfifo : uid",
"optfifo : gid",
"optfifo : mode",
"optfifo : group",
"optfifo : depend",
"setalert : SET alertmail '{' eventoptionlist '}' formatlist reminder",
"setalert : SET alertmail formatlist reminder",
"setalert : SET alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"setdaemon : SET DAEMON NUMBER",
"setinit : SET INIT",
"setlog : SET LOGFILE PATH",
"setlog : SET LOGFILE SYSLOG",
"setlog : SET LOGFILE SYSLOG FACILITY STRING",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH SLOT NUMBER",
"seteventqueue : SET EVENTQUEUE SLOT NUMBER",
"setstate : SET STATEFILE PATH",
"setpid : SET PIDFILE PATH",
"setcollectors : SET COLLECTOR collectorlist",
"collectorlist : collector",
"collectorlist : collectorlist collector",
"collector : URLOBJECT nettimeout sslversion certmd5",
"setmailservers : SET MAILSERVER mailserverlist nettimeout",
"setmailformat : SET MAILFORMAT '{' formatoptionlist '}'",
"sethttpd : SET HTTPD PORT NUMBER httpdlist",
"mailserverlist : mailserver",
"mailserverlist : mailserverlist mailserver",
"mailserver : STRING username password sslversion certmd5",
"mailserver : STRING PORT NUMBER username password sslversion certmd5",
"httpdlist :",
"httpdlist : httpdlist httpdoption",
"httpdoption : ssl",
"httpdoption : signature",
"httpdoption : bindaddress",
"httpdoption : allow",
"ssl : ssldisable",
"ssl : sslenable pemfile clientpemfile allowselfcert",
"sslenable : HTTPDSSL",
"sslenable : HTTPDSSL ENABLE",
"sslenable : ENABLE HTTPDSSL",
"ssldisable : HTTPDSSL DISABLE",
"ssldisable : DISABLE HTTPDSSL",
"ssldisable : ssldisable PEMFILE PATH",
"ssldisable : ssldisable CLIENTPEMFILE PATH",
"ssldisable : ssldisable ALLOWSELFCERTIFICATION",
"signature : sigenable",
"signature : sigdisable",
"sigenable : SIGNATURE ENABLE",
"sigenable : ENABLE SIGNATURE",
"sigdisable : SIGNATURE DISABLE",
"sigdisable : DISABLE SIGNATURE",
"bindaddress : ADDRESS STRING",
"pemfile : PEMFILE PATH",
"clientpemfile :",
"clientpemfile : CLIENTPEMFILE PATH",
"allowselfcert :",
"allowselfcert : ALLOWSELFCERTIFICATION",
"allow : ALLOW STRING ':' STRING readonly",
"allow : ALLOW PATH",
"allow : ALLOW CLEARTEXT PATH",
"allow : ALLOW MD5HASH PATH",
"allow : ALLOW CRYPT PATH",
"$$1 :",
"allow : ALLOW PATH $$1 allowuserlist",
"$$2 :",
"allow : ALLOW CLEARTEXT PATH $$2 allowuserlist",
"$$3 :",
"allow : ALLOW MD5HASH PATH $$3 allowuserlist",
"$$4 :",
"allow : ALLOW CRYPT PATH $$4 allowuserlist",
"allow : ALLOW STRING",
"allowuserlist : allowuser",
"allowuserlist : allowuserlist allowuser",
"allowuser : STRING",
"readonly :",
"readonly : READONLY",
"checkproc : CHECKPROC SERVICENAME PIDFILE PATH",
"checkproc : CHECKPROC SERVICENAME PATHTOK PATH",
"checkfile : CHECKFILE SERVICENAME PATHTOK PATH",
"checkdev : CHECKDEV SERVICENAME PATHTOK PATH",
"checkdir : CHECKDIR SERVICENAME PATHTOK PATH",
"checkhost : CHECKHOST SERVICENAME ADDRESS STRING",
"checksystem : CHECKSYSTEM SERVICENAME",
"checkfifo : CHECKFIFO SERVICENAME PATHTOK PATH",
"start : START argumentlist",
"start : START argumentlist useroptionlist",
"stop : STOP argumentlist",
"stop : STOP argumentlist useroptionlist",
"argumentlist : argument",
"argumentlist : argumentlist argument",
"useroptionlist : useroption",
"useroptionlist : useroptionlist useroption",
"argument : STRING",
"argument : PATH",
"useroption : UID STRING",
"useroption : GID STRING",
"useroption : UID NUMBER",
"useroption : GID NUMBER",
"username :",
"username : USERNAME STRING",
"password :",
"password : PASSWORD STRING",
"connection : IF FAILED host port type protocol nettimeout rate1 THEN action1 recovery",
"connection : IF FAILED URL URLOBJECT urloption nettimeout rate1 THEN action1 recovery",
"connectionunix : IF FAILED unixsocket type protocol nettimeout rate1 THEN action1 recovery",
"icmp : IF FAILED ICMP icmptype icmpcount nettimeout rate1 THEN action1 recovery",
"host :",
"host : HOST STRING",
"port : PORT NUMBER",
"unixsocket : UNIXSOCKET PATH",
"type :",
"type : TYPE TCP",
"type : TYPE TCPSSL sslversion certmd5",
"type : TYPE UDP",
"certmd5 :",
"certmd5 : CERTMD5 STRING",
"sslversion :",
"sslversion : SSLV2",
"sslversion : SSLV3",
"sslversion : TLSV1",
"sslversion : SSLAUTO",
"protocol :",
"protocol : PROTOCOL APACHESTATUS apache_stat_list",
"protocol : PROTOCOL DEFAULT",
"protocol : PROTOCOL DNS",
"protocol : PROTOCOL DWP",
"protocol : PROTOCOL FTP",
"protocol : PROTOCOL HTTP request",
"protocol : PROTOCOL IMAP",
"protocol : PROTOCOL CLAMAV",
"protocol : PROTOCOL LDAP2",
"protocol : PROTOCOL LDAP3",
"protocol : PROTOCOL MYSQL",
"protocol : PROTOCOL NNTP",
"protocol : PROTOCOL NTP3",
"protocol : PROTOCOL POSTFIXPOLICY",
"protocol : PROTOCOL POP",
"protocol : PROTOCOL SMTP",
"protocol : PROTOCOL SSH",
"protocol : PROTOCOL RDATE",
"protocol : PROTOCOL RSYNC",
"protocol : PROTOCOL TNS",
"protocol : PROTOCOL PGSQL",
"protocol : sendexpectlist",
"sendexpectlist : sendexpect",
"sendexpectlist : sendexpectlist sendexpect",
"sendexpect : SEND STRING",
"sendexpect : EXPECT STRING",
"request :",
"request : REQUEST PATH",
"request : REQUEST PATH CHECKSUM STRING",
"apache_stat_list : apache_stat",
"apache_stat_list : apache_stat_list OR apache_stat",
"apache_stat : LOGLIMIT operator NUMBER PERCENT",
"apache_stat : CLOSELIMIT operator NUMBER PERCENT",
"apache_stat : DNSLIMIT operator NUMBER PERCENT",
"apache_stat : KEEPALIVELIMIT operator NUMBER PERCENT",
"apache_stat : REPLYLIMIT operator NUMBER PERCENT",
"apache_stat : REQUESTLIMIT operator NUMBER PERCENT",
"apache_stat : STARTLIMIT operator NUMBER PERCENT",
"apache_stat : WAITLIMIT operator NUMBER PERCENT",
"apache_stat : GRACEFULLIMIT operator NUMBER PERCENT",
"apache_stat : CLEANUPLIMIT operator NUMBER PERCENT",
"pid : IF CHANGED PID rate1 THEN action1",
"ppid : IF CHANGED PPID rate1 THEN action1",
"icmpcount :",
"icmpcount : COUNT NUMBER",
"nettimeout :",
"nettimeout : TIMEOUT NUMBER SECOND",
"timeout : IF NUMBER RESTART NUMBER CYCLE THEN TIMEOUT",
"urloption :",
"urloption : CONTENT urloperator STRING",
"urloperator : EQUAL",
"urloperator : NOTEQUAL",
"alert : alertmail '{' eventoptionlist '}' formatlist reminder",
"alert : alertmail formatlist reminder",
"alert : alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"alert : noalertmail",
"alertmail : ALERT MAILADDR",
"noalertmail : NOALERT MAILADDR",
"eventoptionlist : eventoption",
"eventoptionlist : eventoptionlist eventoption",
"eventoption : CHANGED",
"eventoption : CHECKSUM",
"eventoption : CONNECTION",
"eventoption : DATA",
"eventoption : EXEC",
"eventoption : GID",
"eventoption : ICMP",
"eventoption : INSTANCE",
"eventoption : INVALID",
"eventoption : MATCH",
"eventoption : NONEXIST",
"eventoption : PERMISSION",
"eventoption : RESOURCE",
"eventoption : SIZE",
"eventoption : TIMEOUT",
"eventoption : TIMESTAMP",
"eventoption : UID",
"eventoption : START",
"eventoption : STOP",
"eventoption : RESTART",
"eventoption : UNMONITOR",
"formatlist :",
"formatlist : MAILFORMAT '{' formatoptionlist '}'",
"formatoptionlist : formatoption",
"formatoptionlist : formatoptionlist formatoption",
"formatoption : MAILFROM",
"formatoption : MAILSUBJECT",
"formatoption : MAILBODY",
"every : EVERY NUMBER CYCLE",
"mode : MODE ACTIVE",
"mode : MODE PASSIVE",
"mode : MODE MANUAL",
"group : GROUP STRINGNAME",
"depend : DEPENDS dependlist",
"dependlist : dependant",
"dependlist : dependlist dependant",
"dependant : SERVICENAME",
"resourceprocess : IF resourceprocesslist rate1 THEN action1 recovery",
"resourceprocesslist : resourceprocessopt",
"resourceprocesslist : resourceprocesslist resourceprocessopt",
"resourceprocessopt : resourcecpuproc",
"resourceprocessopt : resourcemem",
"resourceprocessopt : resourcechild",
"resourceprocessopt : resourceload",
"resourcesystem : IF resourcesystemlist rate1 THEN action1 recovery",
"resourcesystemlist : resourcesystemopt",
"resourcesystemlist : resourcesystemlist resourcesystemopt",
"resourcesystemopt : resourceload",
"resourcesystemopt : resourcemem",
"resourcesystemopt : resourcecpu",
"resourcecpuproc : CPU operator NUMBER PERCENT",
"resourcecpu : resourcecpuid operator NUMBER PERCENT",
"resourcecpuid : CPUUSER",
"resourcecpuid : CPUSYSTEM",
"resourcecpuid : CPUWAIT",
"resourcemem : MEMORY operator value unit",
"resourcemem : MEMORY operator NUMBER PERCENT",
"resourcemem : TOTALMEMORY operator value unit",
"resourcemem : TOTALMEMORY operator NUMBER PERCENT",
"resourcechild : CHILDREN operator NUMBER",
"resourceload : resourceloadavg operator value",
"resourceloadavg : LOADAVG1",
"resourceloadavg : LOADAVG5",
"resourceloadavg : LOADAVG15",
"value : REAL",
"value : NUMBER",
"timestamp : IF TIMESTAMP operator NUMBER time rate1 THEN action1 recovery",
"timestamp : IF CHANGED TIMESTAMP rate1 THEN action1",
"operator :",
"operator : GREATER",
"operator : LESS",
"operator : EQUAL",
"operator : NOTEQUAL",
"operator : CHANGED",
"time :",
"time : SECOND",
"time : MINUTE",
"time : HOUR",
"time : DAY",
"action : ALERT",
"action : EXEC argumentlist",
"action : EXEC argumentlist useroptionlist",
"action : RESTART",
"action : START",
"action : STOP",
"action : UNMONITOR",
"action1 : action",
"action2 : action",
"rate1 :",
"rate1 : NUMBER CYCLE",
"rate1 : NUMBER NUMBER CYCLE",
"rate2 :",
"rate2 : NUMBER CYCLE",
"rate2 : NUMBER NUMBER CYCLE",
"recovery :",
"recovery : ELSE IF RECOVERED rate2 THEN action2",
"recovery : ELSE IF PASSED rate2 THEN action2",
"checksum : IF FAILED hashtype CHECKSUM rate1 THEN action1 recovery",
"checksum : IF FAILED hashtype CHECKSUM EXPECT STRING rate1 THEN action1 recovery",
"checksum : IF CHANGED hashtype CHECKSUM rate1 THEN action1",
"hashtype :",
"hashtype : MD5HASH",
"hashtype : SHA1HASH",
"inode : IF INODE operator NUMBER rate1 THEN action1 recovery",
"inode : IF INODE operator NUMBER PERCENT rate1 THEN action1 recovery",
"space : IF SPACE operator value unit rate1 THEN action1 recovery",
"space : IF SPACE operator NUMBER PERCENT rate1 THEN action1 recovery",
"fsflag : IF CHANGED FSFLAG rate1 THEN action1",
"unit : BYTE",
"unit : KILOBYTE",
"unit : MEGABYTE",
"unit : GIGABYTE",
"permission : IF FAILED PERMISSION NUMBER rate1 THEN action1 recovery",
"match : IF matchflagnot MATCH PATH rate1 THEN action1",
"match : IF matchflagnot MATCH STRING rate1 THEN action1",
"match : IGNORE matchflagnot MATCH PATH",
"match : IGNORE matchflagnot MATCH STRING",
"matchflagnot :",
"matchflagnot : NOT",
"size : IF SIZE operator NUMBER unit rate1 THEN action1 recovery",
"size : IF CHANGED SIZE rate1 THEN action1",
"uid : IF FAILED UID STRING rate1 THEN action1 recovery",
"uid : IF FAILED UID NUMBER rate1 THEN action1 recovery",
"gid : IF FAILED GID STRING rate1 THEN action1 recovery",
"gid : IF FAILED GID NUMBER rate1 THEN action1 recovery",
"icmptype : TYPE ICMPECHO",
"reminder :",
"reminder : REMINDER NUMBER",
"reminder : REMINDER NUMBER CYCLE",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH  500
#endif
#endif

#define YYINITSTACKSIZE 500

int      yydebug;
int      yynerrs;
int      yyerrflag;
int      yychar;
short   *yyssp;
YYSTYPE *yyvsp;
YYSTYPE  yyval;
YYSTYPE  yylval;

/* variables for the parser stack */
static short   *yyss;
static short   *yysslim;
static YYSTYPE *yyvs;
static int      yystacksize;
#line 1554 "p.y"


/* -------------------------------------------------------- Parser interface */


/**
 * Syntactic error routine
 *
 * This routine is automatically called by the lexer!
 */
void yyerror(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg= NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg= Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", currentfile, lineno, msg, yytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Syntactical warning routine
 */
void yywarning(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg= NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg= Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", currentfile, lineno, msg, yytext);
 
  FREE(msg);
  
}

/**
 * Argument error routine
 */
void yyerror2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg= NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg= Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Argument warning routine
 */
void yywarning2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg= NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg= Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
 
  FREE(msg);
  
}

/*
 * The Parser hook - start parsing the control file
 * Returns TRUE if parsing succeeded, otherwise FALSE
 */
int parse(char *controlfile) {

  ASSERT(controlfile);

  servicelist= tail= current= NULL;

  /*
   * Secure check the monitrc file. The run control file must have the
   * same uid as the REAL uid of this process, it must have permissions
   * no greater than 700 and it must not be a symbolic link.
   */
  if (! File_checkStat(controlfile, "control file", S_IRUSR|S_IWUSR|S_IXUSR)) {
    return FALSE;
  }

  if ((yyin = fopen(controlfile,"r")) == (FILE *)NULL) {
    LogError("%s: Error: cannot open the control file '%s' -- %s\n",
	prog, controlfile, STRERROR);
    return FALSE;
  }

  currentfile=xstrdup(controlfile);

  /* 
   * Creation of the global service list is synchronized  
   */
  LOCK(Run.mutex)
    preparse();
    yyparse();
    fclose(yyin);
    /* Add the default general system service if not specified explicitly */
    if(!hassystem) {
      createservice(TYPE_SYSTEM, xstrdup(Run.localhostname), xstrdup(""), check_system);
    }
    /* If defined - add the last service to the service list */
    if(current) {
      addservice(current);
      FREE(current);
    }
    postparse();
  END_LOCK;

  FREE(currentfile);

  if (argyytext!=NULL)
      FREE(argyytext);

  return(cfg_errflag == 0);
}


/* ----------------------------------------------------------------- Private */


/**
 * Initialize objects used by the parser.
 */
static void preparse() {
  int i;
  char localhost[STRLEN];

  /*
   * Get the localhost name
   */
  if(gethostname(localhost, sizeof(localhost)) < 0) {
    snprintf(localhost, STRLEN, "%s", LOCALHOST);
  }

  /* Set instance incarnation ID */
  time(&Run.incarnation);
  /* Reset lexer */
  buffer_stack_ptr=0;
  lineno= 1;
  arglineno= 1;
  argcurrentfile=NULL;
  argyytext=NULL;
  /* Reset parser */
  Run.wait_start= 0;
  Run.stopped= FALSE;
  Run.dolog= FALSE;
  Run.dohttpd= FALSE;
  Run.doaction= FALSE;
  Run.httpdsig= TRUE;
  Run.credentials= NULL;
  Run.httpdssl= FALSE;
  Run.httpsslpem= NULL;
  Run.httpsslclientpem= NULL;
  Run.clientssl= FALSE;
  Run.mailserver_timeout= NET_TIMEOUT;
  Run.bind_addr= NULL;
  Run.eventlist= NULL;
  Run.eventlist_dir= NULL;
  Run.eventlist_slots= -1;
  Run.system= NULL;
  Run.maillist= NULL;
  Run.mailservers= NULL;
  Run.MailFormat.from= NULL;
  Run.MailFormat.subject= NULL;
  Run.MailFormat.message= NULL;
  Run.localhostname= xstrdup(localhost);
  depend_list= NULL;
  Run.handler_init= TRUE;
  for(i=0; i<=HANDLER_MAX; i++) {
    Run.handler_queue[i] = 0;
  }
  /* 
   * Initialize objects
   */
  reset_uidset();
  reset_gidset();
  reset_sizeset();
  reset_mailset();
  reset_mailserverset();
  reset_portset();
  reset_permset();
  reset_icmpset();
  reset_rateset();
  reset_deviceset();
  reset_resourceset();
  reset_checksumset();
  reset_timestampset();
}


/*
 * Check that values are reasonable after parsing
 */
static void postparse() {

  Service_T s;

  if(cfg_errflag || (servicelist==NULL)) {
    return;
  }

  /* Check the sanity of any dependency graph */
  check_depend();

  /* Check that we do not start monit in daemon mode without having a
   * poll time */
  if(!Run.polltime && (Run.isdaemon || Run.init)) {
    LogError("%s: Error: Poll time not defined. Please define poll time"
	" in the\n control file or use the -d option when starting monit\n",
	prog);
    cfg_errflag++;
  }

  if(Run.logfile) {
    Run.dolog = TRUE;
  }

  for(s= servicelist; s; s= s->next) {
    /* Set the general system service shortcut */
    if(s->type == TYPE_SYSTEM)
      Run.system = s;
    if(s->type != TYPE_HOST)
	continue;
    /* Verify that a remote service has a port or an icmp list */
    if(!s->portlist && !s->icmplist) {
      LogError("%s: Error: 'check host' statement is incomplete; Please"
          " specify a port number to test\n or an icmp test at the remote"
          " host: '%s'\n",
	  prog, s->name);
      cfg_errflag++;
    }
  }
}


/*
 * Create a new service object and add any current objects to the
 * service list.
 */
static void createservice(int type, char *name, char *value,
			  int (*check)(Service_T s)) {

  ASSERT(name);
  ASSERT(value);

  if(current) {
    addservice(current);
  } else {
    NEW(current);
  }

  /* Reset the current object */
  memset(current, 0, sizeof(*current));

  NEW(current->inf);
  Util_resetInfo(current);

  /* Set default values */
  current->monitor= MONITOR_INIT;
  current->mode= MODE_ACTIVE;

  current->name= name;
  current->type= type;
  current->check= check;
  current->path= value;

  /* Initialize general event handlers */
  addeventaction(&(current)->action_DATA,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_EXEC,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_INVALID,  ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_NONEXIST, ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_TIMEOUT,  ACTION_UNMONITOR, ACTION_ALERT);
  addeventaction(&(current)->action_PID,      ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_PPID,     ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_FSFLAG,   ACTION_ALERT,     ACTION_IGNORE);

  /* Initialize internal event handlers */
  addeventaction(&(current)->action_MONIT_START,  ACTION_ALERT, ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_STOP,   ACTION_ALERT, ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_RELOAD, ACTION_ALERT, ACTION_IGNORE);
  
  pthread_mutex_init(&current->mutex, NULL);

  time(&current->collected);

}


/*
 * Add a service object to the servicelist
 */
static void addservice(Service_T s) {
  
  Service_T n;

  ASSERT(s);
 
  NEW(n);
  memcpy(n, s, sizeof(*s));

  /* Add the service to the end of the service list */
  if(tail != NULL) {
    
    tail->next= n;
    tail->next_conf= n;
    
  } else {
    
    servicelist= n;
    servicelist_conf= n;
    
  }
  
  tail= n;

}


/* 
 * Add a dependant entry to the current service dependant list
 *
 */
static void adddependant(char *dependant) {

  Dependant_T d; 

  ASSERT(dependant);
  
  NEW(d);
  
  if (current->dependantlist != NULL) {
    d->next= current->dependantlist;
  }

  d->dependant=dependant;
  current->dependantlist= d;

}


/*
 * Add the given mailaddress with the apropriate alert notification
 * values and mail attributes to the given mailinglist.
 */
static void addmail(char *mailto, Mail_T f, Mail_T *l, unsigned int events, unsigned int reminder) {

  Mail_T m;

  ASSERT(mailto);

  NEW(m);
  m->events= events;
  m->to= mailto;
  m->from= f->from;
  m->subject= f->subject;
  m->message= f->message;
  m->reminder= reminder;
  
  m->next= *l;
  *l= m;

  reset_mailset();

}


/*
 * Add the given portset to the current service's portlist
 */
static void addport(Port_T port) {

  Port_T p;
  char address[STRLEN];
  
  ASSERT(port);

  NEW(p);
  p->port= port->port;
  p->type= port->type;
  p->socket= port->socket;
  p->family= port->family;
  p->action= port->action;
  p->timeout= port->timeout;
  p->request= port->request;
  p->generic= port->generic;
  p->protocol= port->protocol;
  p->pathname= port->pathname;
  p->hostname= port->hostname;
  p->url_request= port->url_request;
  p->request_checksum= port->request_checksum;
  memcpy(&p->ApacheStatus, &port->ApacheStatus, sizeof(struct apache_status));

  if (p->request_checksum) {
    cleanup_hash_string(p->request_checksum);
    if (strlen(p->request_checksum)==32) {
      p->request_hashtype=HASH_MD5;
    } else if (strlen(p->request_checksum)==40) {
      p->request_hashtype=HASH_SHA1;
    } else {
      yyerror2("invalid checksum [%s]", p->request_checksum);
    }
  } else {
    p->request_hashtype=0;
  }

  if(port->family == AF_INET) {
    snprintf(address, STRLEN, "INET[%s:%d]", port->hostname, port->port);
  } else if(port->family == AF_UNIX) {
    snprintf(address, STRLEN, "UNIX[%s]", port->pathname);
  }
  p->address= xstrdup(address);
  
  if(port->SSL.use_ssl == TRUE) {
    if(!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      if (port->SSL.certmd5 != NULL) {
	p->SSL.certmd5= port->SSL.certmd5;
	cleanup_hash_string(p->SSL.certmd5);
      }
      p->SSL.use_ssl= TRUE;
      p->SSL.version= port->SSL.version;
    }
  }
  
  p->next= current->portlist;
  current->portlist= p;
  
  reset_portset();

}


/*
 * Add a new resource object to the current service resource list
 */
static void addresource(Resource_T rr) {

  Resource_T r;

  ASSERT(rr);

  NEW(r);
  if(! Run.doprocess) {
    yyerror("cannot activate service check.\n"
            "\t(The process status engine was disabled. On certain"
            " systems you must\n\trun monit as root to utilize this"
            " feature)\n\t");
  }
  r->resource_id= rr->resource_id;
  r->limit= rr->limit;
  r->action= rr->action;
  r->operator= rr->operator;
  r->next= current->resourcelist;

  current->resourcelist= r;
  reset_resourceset();
}


/*
 * Add a new file object to the current service timestamp list
 */
static void addtimestamp(Timestamp_T ts, int notime) {

  Timestamp_T t;

  ASSERT(ts);

  NEW(t);
  t->operator= ts->operator;
  t->time= ts->time;
  t->action= ts->action;
  t->test_changes= ts->test_changes;
  
  if(t->test_changes || notime) {
    if(!File_exist(current->path)) {
      DEBUG("%s: Debug: the path '%s' used in the TIMESTAMP statement"
            " refer to a non-existing object\n", prog, current->path);
    } else if(!(t->timestamp= 
		File_getTimestamp(current->path, S_IFDIR|S_IFREG))) {
	    yyerror2("cannot get the timestamp for '%s'", current->path);
    }
  }
  
  t->next= current->timestamplist;
  current->timestamplist= t;

  reset_timestampset();

}


/*
 * Add a new Size object to the current service size list
 */
static void addsize(Size_T ss, int nosize) {

  Size_T s;
  struct stat buf;

  ASSERT(ss);

  /*
     In the case that we are testig changes, get the initial
     size for future comparision. In the case that we are not
     testing changes, we don't need the initial size nor require
     file to exist in the time of configuration file parsing
  */
  if(ss->test_changes != 0 && stat(current->path, &buf) != 0) {
    yyerror2("cannot get size for '%s'", current->path);
    reset_sizeset();
    return;
  }
 
  NEW(s);
  s->operator= ss->operator;
  s->size= ss->size;
  s->action= ss->action;
  s->test_changes= ss->test_changes;
  if(ss->test_changes || nosize)
    s->size= (unsigned long long)buf.st_size;
 
  s->next= current->sizelist;
  current->sizelist= s;

  reset_sizeset();
}


/*
 * Set Checksum object in the current service
 */
static void addchecksum(Checksum_T cs) {

  int len;
  Checksum_T c;

  ASSERT(cs);

  if(!cs->hash) {
    if(cs->type == HASH_UNKNOWN) {
      cs->type=DEFAULT_HASH;
    }
    if( !(cs->hash= Util_getChecksum(current->path, cs->type))) {
      yyerror2("cannot compute a checksum for file %s", current->path);
      reset_checksumset();
      return;
    }
  }

  len= cleanup_hash_string(cs->hash);

  if(cs->type == HASH_UNKNOWN) {
    if (len==32) {
      cs->type=HASH_MD5;
    } else if (len==40) {
      cs->type=HASH_SHA1;
    } else {
      yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
      reset_checksumset();
      return;
    }
  } else if (( cs->type==HASH_MD5 && len!=32 ) ||
             ( cs->type==HASH_SHA1 && len!=40 )) {
    yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
    reset_checksumset();
    return;
  }

  NEW(c);

  c->type=cs->type;
  c->hash= cs->hash;
  c->test_changes= cs->test_changes;
  c->action= cs->action;
 
  current->checksum= c;

  reset_checksumset();

}


/*
 * Set Perm object in the current service
 */
static void addperm(Perm_T ps) {

  Perm_T p;

  ASSERT(ps);

  NEW(p);
  p->perm= ps->perm;
  p->action= ps->action;
  current->perm= p;
  reset_permset();

}

/*
 * Set Match object in the current service
 */
static void addmatch(Match_T ms, int actionnumber, int linenumber) {

  Match_T m;
  Match_T ml;
  int     reg_return;
  
  ASSERT(ms);

  NEW(m);
#ifdef HAVE_REGEX_H
  NEW(m->regex_comp);
#endif

  m->match_string= ms->match_string;
  m->match_path= ms->match_path?xstrdup(ms->match_path):NULL;
  m->action= ms->action;
  m->not= ms->not;
  m->ignore= ms->ignore;
  m->next=NULL;

  addeventaction(&(m->action), actionnumber, ACTION_IGNORE);

#ifdef HAVE_REGEX_H
  reg_return= regcomp(m->regex_comp, ms->match_string, REG_NOSUB|REG_EXTENDED);

  if (reg_return!=0) {
    char errbuf[STRLEN];
    regerror(reg_return, ms->regex_comp, errbuf, STRLEN);
    if (m->match_path != NULL) 
        yyerror2("regex parsing error:%s on line %i of", errbuf, linenumber);
    else
        yyerror2("regex parsing error:%s", errbuf);
  }
#endif

  if (current->matchlist) {
    /* Find the end of the list */
    for(ml=current->matchlist; ml->next; ml=ml->next);

    ml->next= m;
    
  } else {

    current->matchlist= m;
        
  }
}


static void addmatchpath(Match_T ms, int actionnumber) {

  FILE *handle;
  Command_T savecommand = NULL;
  char buf[2048];
  int linenumber=0;

  ASSERT(ms->match_path);

  handle = fopen(ms->match_path, "r");
  if(handle == NULL) {
    yyerror2("cannot read regex match file (%s)", ms->match_path);
    return;
  }
  
  while(!feof(handle)) {
    linenumber++;
    
    if(! fgets(buf, 2048, handle))
      continue;

    if(strlen(buf)==0 || buf[0]=='\n')
      continue;

    if(buf[strlen(buf)-1] == '\n')
      buf[strlen(buf)-1] = 0;

    ms->match_string = xstrdup(buf);

    /* The addeventaction() called from addmatch() will reset the
     * command1 to NULL, but we need to duplicate the command for
     * each line, thus need to save it here */
    if(actionnumber == ACTION_EXEC) {
      if(command1 == NULL) {
        ASSERT(savecommand);
        command1 = savecommand;
      }
      ASSERT(command1);
      savecommand = copycommand(command1);
    }

    addmatch(ms, actionnumber, linenumber);
  }

  if(actionnumber == ACTION_EXEC && savecommand)
    _gccmd(&savecommand);

  fclose(handle);
}


/*
 * Set Uid object in the current service
 */
static void adduid(Uid_T us) {

  Uid_T u;

  ASSERT(us);

  NEW(u);
  u->uid= us->uid;
  u->action= us->action;
  current->uid= u;
  reset_uidset();

}


/*
 * Set Gid object in the current service
 */
static void addgid(Gid_T gs) {

  Gid_T g;

  ASSERT(gs);

  NEW(g);
  g->gid= gs->gid;
  g->action= gs->action;
  current->gid= g;
  reset_gidset();

}


/*
 * Add a new device to the current service's device list
 */
static void adddevice(Device_T ds) {

  Device_T dev;

  ASSERT(ds);
  
  NEW(dev);
  dev->resource= ds->resource;
  dev->operator= ds->operator;
  dev->limit_absolute= ds->limit_absolute;
  dev->limit_percent= ds->limit_percent;
  dev->action= ds->action;

  dev->next= current->devicelist;
  current->devicelist= dev;

  reset_deviceset();

}


/*
 * Add a new icmp object to the current service's icmp list
 */
static void addicmp(Icmp_T is) {
  if(!getuid()) {

    Icmp_T icmp;

    ASSERT(is);

    NEW(icmp);
    icmp->type= is->type;      
    icmp->count= is->count > ICMP_MAX_COUNT ? ICMP_MAX_COUNT : is->count;     
    icmp->timeout= is->timeout;
    icmp->action= is->action;
    icmp->is_available= FALSE;
    icmp->response= -1;
  
    icmp->next= current->icmplist;
    current->icmplist= icmp;

  } else {
    yyerror("icmp statements must be run as root");
  }
  reset_icmpset();
}


/*
 * Set EventAction object
 */
static void addeventaction(EventAction_T *_ea, int failed, int passed) {

  EventAction_T ea;

  ASSERT(_ea);

  NEW(ea);
  NEW(ea->failed);
  NEW(ea->passed);

  ea->failed->id= failed;
  ea->failed->count= rate1.count;
  ea->failed->cycles= rate1.cycles;
  if(failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }
  describeAction(ea->failed);

  ea->passed->id= passed;
  ea->passed->count= rate2.count;
  ea->passed->cycles= rate2.cycles;
  if(passed == ACTION_EXEC) {
    ASSERT(command2);
    ea->passed->exec = command2;
    command2 = NULL;
  }
  describeAction(ea->passed);
 
  *_ea= ea;

  reset_rateset();

}


/*
 * Redefine EventAction object (used for default action overloading)
 */
static void seteventaction(EventAction_T *_ea, int failed, int passed) {

  EventAction_T ea = *_ea;

  ASSERT(ea);
  ASSERT(ea->failed);
  ASSERT(ea->passed);

  ea->failed->id= failed;
  ea->failed->count= rate1.count;
  ea->failed->cycles= rate1.cycles;
  if(failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }
  describeAction(ea->failed);

  ea->passed->id= passed;
  ea->passed->count= rate2.count;
  ea->passed->cycles= rate2.cycles;
  if(passed == ACTION_EXEC) {
    ASSERT(command2);
    ea->passed->exec = command2;
    command2 = NULL;
  }
  describeAction(ea->passed);
}


/*
 * Return a protocol object for the given protocol
 */
static void *addprotocol(int protocol) {

  switch (protocol) {
  case P_APACHESTATUS:  return create_apache_status();
  case P_DNS:           return create_dns();
  case P_DWP:           return create_dwp();
  case P_FTP:           return create_ftp();
  case P_GENERIC:       return create_generic();
  case P_HTTP:          return create_http();
  case P_IMAP:          return create_imap();
  case P_CLAMAV:        return create_clamav();
  case P_LDAP2:         return create_ldap2();
  case P_LDAP3:         return create_ldap3();
  case P_MYSQL:         return create_mysql();
  case P_NNTP:          return create_nntp();
  case P_NTP3:          return create_ntp3();
  case P_POSTFIXPOLICY: return create_postfix_policy();
  case P_POP:           return create_pop();
  case P_SMTP:          return create_smtp();
  case P_SSH:           return create_ssh();
  case P_RDATE:         return create_rdate();
  case P_RSYNC:         return create_rsync();
  case P_TNS:           return create_tns();
  case P_PGSQL:         return create_pgsql();
  }

  return create_default();
   
}


/*
 * Add a generic protocol handler to 
 */
static void addgeneric(Port_T port, char *send, char *expect) {

  Generic_T g= port->generic;
  
  if (g == NULL) {
    NEW(g);
    port->generic= g;
  } else {
    while (g->next != NULL) {
      g= g->next;
    }
    NEW(g->next);
    g=g->next;
  }
  
  if (send != NULL) {
    g->send=xstrdup(send);
    g->expect=NULL;
  } else if (expect != NULL) {
#ifdef HAVE_REGEX_H
    
    int   reg_return;
    NEW(g->expect);
    reg_return= regcomp(g->expect, expect, REG_NOSUB|REG_EXTENDED);
    if (reg_return!=0) {
      char errbuf[STRLEN];
      regerror(reg_return, g->expect, errbuf, STRLEN);
      yyerror2("regex parsing error:%s", errbuf);
    }
#else
    g->expect=xstrdup(expect);
#endif
    g->send=NULL;
  } 
}


/*
 * Add the current command object to the current service object's
 * start or stop program.
 */
static void addcommand(int what) {

  switch(what) {
  case START: current->start= command; break;
  case STOP:  current->stop= command; break;
  }
  
  command= NULL;
  
}

  
/*
 * Add a new argument to the argument list
 */
static void addargument(char *argument) {

  ASSERT(argument);

  if(! command) {
    
    NEW(command);
    check_exec(argument);
    command->timeout = 1;
    
  }
  
  command->arg[command->length++]= argument;
  command->arg[command->length]= NULL;
  
  if(command->length >= ARGMAX) {
    
    yyerror("exceeded maximum number of program arguments");
    
  }
  
}


/*
 * Setup a url request for the current port object
 */
static void prepare_urlrequest(URL_T U) {

  ASSERT(U);
  
  portset.protocol= addprotocol(P_HTTP);

  if(urlrequest==NULL)
    NEW(urlrequest);
  urlrequest->url= U;
  portset.hostname= xstrdup(U->hostname);
  check_hostname(portset.hostname);
  portset.port= U->port;
  portset.url_request= urlrequest;
  portset.type= SOCK_STREAM;
  portset.request= Util_getString("%s%s%s", U->path, U->query?"?":"",
				  U->query?U->query:"");
  /* Only the HTTP protocol is supported for URLs.
     See also the lexer if this is to be changed in
     the future */
  portset.protocol= addprotocol(P_HTTP);
  if(IS(U->protocol, "https")) {
    portset.SSL.use_ssl= TRUE;
  }
  
}


/*
 * Set the url request for a port
 */
static void  seturlrequest(int operator, char *regex) {
  
  ASSERT(regex);

  if(! urlrequest)
    NEW(urlrequest);
  urlrequest->operator= operator;
#ifdef HAVE_REGEX_H
  {    
    int reg_return;
    NEW(urlrequest->regex);
    reg_return= regcomp(urlrequest->regex, regex, REG_NOSUB|REG_EXTENDED);
    if (reg_return!=0) {
      char errbuf[STRLEN];
      regerror(reg_return, urlrequest->regex, errbuf, STRLEN);
      yyerror2("regex parsing error: %s", errbuf);
    }
  }
#else
  urlrequest->regex= xstrdup(regex);
#endif

}


/*
 * Add a new data recipient server to the collector server list
 */
static void addcollector(URL_T url, int timeout, int sslversion, 
			 char *certmd5) {

  Collector_T c;
  
  ASSERT(url);

  NEW(c);
  c->url = url;
  if(!strcmp(c->url->protocol, "https")) {
    if(!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      c->ssl.use_ssl= TRUE;
      c->ssl.version= sslversion;
      if (certmd5) {
	c->ssl.certmd5= certmd5;
	cleanup_hash_string(c->ssl.certmd5);
      }
    }
  }
  c->timeout = timeout;
  c->next = NULL;

  if(Run.collectors) {
    Collector_T C;
    for(C = Run.collectors; C->next; C = C->next)
      /* Empty */ ;
    C->next = c;
  }
  else {
    Run.collectors = c;
  }
}


/*
 * Add a new smtp server to the mail server list
 */
static void addmailserver(MailServer_T mailserver) {

  MailServer_T s;
  
  ASSERT(mailserver->host);

  NEW(s);
  s->host        = mailserver->host;
  s->port        = mailserver->port;
  s->username    = mailserver->username;
  s->password    = mailserver->password;
  s->ssl.use_ssl = mailserver->ssl.use_ssl;
  s->ssl.version = mailserver->ssl.version;
  s->ssl.certmd5 = mailserver->ssl.certmd5;

  s->next= NULL;

  if(Run.mailservers) {
    MailServer_T l;
    for(l= Run.mailservers; l->next; l= l->next) /* empty */;
    l->next= s;
  } else {
    Run.mailservers= s;
  }
  
  reset_mailserverset();
}


/*
 * Return uid if found on the system. If the parameter user is NULL
 * the uid parameter is used for looking up the user id on the system,
 * otherwise the user parameter is used.
 */
static uid_t get_uid(char *user, uid_t uid) {

  struct passwd *pwd;

  if(user) {

    pwd= getpwnam(user);

    if(pwd == NULL) {
      yyerror2("requested user not found on the system");

      return(0);
    }

  } else {

    if( (pwd= getpwuid(uid)) == NULL ) {
      yyerror2("requested uid not found on the system");

      return(0);
    }
  }

  return(pwd->pw_uid);

}


/*
 * Return gid if found on the system. If the parameter group is NULL
 * the gid parameter is used for looking up the group id on the system,
 * otherwise the group parameter is used.
 */
static gid_t get_gid(char *group, gid_t gid) {

  struct group *grd;

  if(group) {

    grd= getgrnam(group);

    if(grd == NULL) {
      yyerror2("requested group not found on the system");
      
      return(0);
    }

  } else {

    if( (grd= getgrgid(gid)) == NULL ) {
      yyerror2("requested gid not found on the system");

      return(0);
    }

  }

  return(grd->gr_gid);

}


/*
 * Add a new user id to the current command object.
 */
static void addeuid(uid_t uid) {

  if(!getuid()) {

    command->has_uid= TRUE;
    command->uid= uid;

  } else {
    
    yyerror("uid statement requires root privileges");

  }

}


/*
 * Add a new group id to the current command object.
 */
static void addegid(gid_t gid) {

  if(!getuid()) {

    command->has_gid= TRUE;
    command->gid= gid;

  } else {

    yyerror("gid statement requires root privileges");

  }

}


/*
 * Reset the logfile if changed
 */
static void setlogfile(char *logfile) {

  if(Run.logfile) {
    
    if(IS(Run.logfile, logfile)) {
      
      FREE(logfile);
      return;
      
    } else {
      
      FREE(Run.logfile);
      
    }
  }
  
  Run.logfile= logfile;
 
}


/*
 * Reset the pidfile if changed
 */
static void setpidfile(char *pidfile) {

  if(Run.pidfile) {
    
    if(IS(Run.pidfile, pidfile)) {
      
      FREE(pidfile);
      return;
      
    } else {
      
      FREE(Run.pidfile);
      
    }
  }
  
  Run.pidfile= pidfile;
 
}


/*
 * Read a apache htpasswd file and add credentials found for username
 */
static void addhtpasswdentry(char *filename, char *username, int dtype) {

  char *ht_username= NULL;
  char *ht_passwd= NULL;
  char buf[STRLEN];
  FILE *handle= NULL;
  int credentials_added= 0;
  
  ASSERT(filename);

  handle=fopen(filename, "r");

  if ( handle==NULL ) {

    if (username!=NULL) {
      yyerror2("cannot read htpasswd (%s)", filename);
    } else {
      yyerror2("cannot read htpasswd", filename);
    }
    return;
    
  }
  
  while (!feof(handle)) {
    char *colonindex= NULL;
    int i;
    
    if (! fgets(buf, STRLEN, handle)) {

      continue;
      
    }

    /* strip trailing non visible characters */
    for (i=strlen(buf)-1; i >= 0; i--) {
      if ( buf[i] == ' '  || buf[i] == '\r' ||
           buf[i] == '\n' || buf[i] == '\t' ) {
        
        buf[i]='\0';
        
      } else {
        
        break;
        
      }
    }

    if ( NULL == (colonindex=strchr(buf, ':'))) {

      continue;
      
    }

    ht_passwd=xstrdup(colonindex+1);
    *colonindex='\0';

    /* Incase we have a file in /etc/passwd or /etc/shadow style we
     *  want to remove ":.*$" and Crypt and MD5 hashed dont have a colon
     */ 
    
    if ( (NULL != (colonindex=strchr(ht_passwd, ':'))) &&
         ( dtype != DIGEST_CLEARTEXT) ) {

      *colonindex='\0';
      
    }

    ht_username=xstrdup(buf);

    if (username==NULL) {
      
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE)) {  
        credentials_added++;
      }
      
    } else if (strcmp(username, ht_username) == 0)  {
      
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE)) {
        credentials_added++;
      }
      
    } else {

      FREE(ht_passwd);
      FREE(ht_username);
            
    }
  }

  if (credentials_added==0) {

    if ( username == NULL ) {
      
      yywarning2("htpasswd file (%s) has no usable credentials",
          filename);

    } else {

      yywarning2("htpasswd file (%s) has no usable credentials "
                 "for user %s", filename, username);

    }
    
    
  }

  fclose(handle);
  
}


/*
 * Add Basic Authentication credentials
 */
static int addcredentials(char *uname, char *passwd, int dtype, int readonly) {

  Auth_T c;

  ASSERT(uname);
  ASSERT(passwd);

  if (Run.credentials == NULL) {

    NEW(Run.credentials);
    c=Run.credentials;
    
  } else {

    if (Util_getUserCredentials(uname) != NULL) {

      yywarning2("credentials for user %s were already added, entry ignored",
                 uname);
      FREE(uname);
      FREE(passwd);
      return FALSE;
      
    }

    c=Run.credentials;

    while ( c->next != NULL ) {

      c=c->next;
      
    }

    NEW(c->next);
    c=c->next;
        
  }
  
  c->next=NULL;
  c->uname=uname;
  c->passwd=passwd;
  c->digesttype=dtype;
  c->is_readonly= readonly;
  
  DEBUG("%s: Debug: Adding credentials for user '%s'.\n", prog, uname); 
  
  return TRUE;
  
}


/*
 * Set the syslog and the facilities to be used
 */
static void setsyslog(char *facility) {

  if (!Run.logfile || ihp.logfile) {

    ihp.logfile= TRUE;
    setlogfile(xstrdup("syslog"));
    Run.use_syslog= TRUE;
    Run.dolog=TRUE;

  }

  if(facility) {
    if(IS(facility,"log_local0")) {
      Run.facility = LOG_LOCAL0;
    } else if(IS(facility, "log_local1")) {
      Run.facility = LOG_LOCAL1;
    } else if(IS(facility, "log_local2")) {
      Run.facility = LOG_LOCAL2;
    } else if(IS(facility, "log_local3")) {
      Run.facility = LOG_LOCAL3;
    } else if(IS(facility, "log_local4")) {
      Run.facility = LOG_LOCAL4;
    } else if(IS(facility, "log_local5")) {
      Run.facility = LOG_LOCAL5;
    } else if(IS(facility, "log_local6")) {
      Run.facility = LOG_LOCAL6;
    } else if(IS(facility, "log_local7")) {
      Run.facility = LOG_LOCAL7;
    } else if(IS(facility, "log_daemon")) {
      Run.facility = LOG_DAEMON;
    } else {
      yyerror2("invalid syslog facility");
    }
  } else {
    Run.facility= LOG_USER;
  }
  
}


/*
 * Reset the current mailset, eventset and reminder for reuse
 */
static void reset_mailset() {
  memset(&mailset, 0, sizeof(struct mymail));
  eventset= EVENT_NULL;
}


/*
 * Reset the mailserver set to default values
 */
static void reset_mailserverset() {
  memset(&mailserverset, 0, sizeof(struct mymailserver));
  mailserverset.port = PORT_SMTP;
  mailserverset.ssl.use_ssl = FALSE;
  mailserverset.ssl.version= SSL_VERSION_AUTO;
}


/*
 * Reset the Port set to default values
 */
static void reset_portset() {
  memset(&portset, 0, sizeof(struct myport));
  portset.socket= -1;
  portset.type= SOCK_STREAM;
  portset.family= AF_INET;
  portset.SSL.version= SSL_VERSION_AUTO;
  portset.timeout= NET_TIMEOUT;
  urlrequest= NULL;
}


/*
 * Reset the Proc set to default values
 */
static void reset_resourceset() {
  resourceset.resource_id= 0;
  resourceset.limit= 0;
  resourceset.action= NULL;
  resourceset.operator= OPERATOR_EQUAL;
}


/*
 * Reset the Timestamp set to default values
 */
static void reset_timestampset() {
  timestampset.operator= OPERATOR_EQUAL;
  timestampset.time= 0;
  timestampset.test_changes= FALSE;
  timestampset.action= NULL;
}


/*
 * Reset the Size set to default values
 */
static void reset_sizeset() {
  sizeset.operator= OPERATOR_EQUAL;
  sizeset.size= 0;
  sizeset.test_changes= FALSE;
  sizeset.action= NULL;
}


/*
 * Reset the Checksum set to default values
 */
static void reset_checksumset() {
  checksumset.type= HASH_UNKNOWN;
  checksumset.hash= NULL;
  checksumset.test_changes= FALSE;
  checksumset.action= NULL;
}


/*
 * Reset the Perm set to default values
 */
static void reset_permset() {
  permset.perm= 0;
  permset.action= NULL;
}


/*
 * Reset the Uid set to default values
 */
static void reset_uidset() {
  uidset.uid= 0;
  uidset.action= NULL;
}


/*
 * Reset the Gid set to default values
 */
static void reset_gidset() {
  gidset.gid= 0;
  gidset.action= NULL;
}


/*
 * Reset the Device set to default values
 */
static void reset_deviceset() {
  deviceset.resource= 0;
  deviceset.operator= OPERATOR_EQUAL;
  deviceset.limit_absolute= -1;
  deviceset.limit_percent= -1;
  deviceset.action= NULL;
}


/*
 * Reset the ICMP set to default values
 */
static void reset_icmpset() {
  icmpset.type= ICMP_ECHO;
  icmpset.count= ICMP_ATTEMPT_COUNT;
  icmpset.timeout= NET_TIMEOUT;
  icmpset.action= NULL;
}


/*
 * Reset the Rate set to default values
 */
static void reset_rateset() {
  rate1.count  = 1;
  rate1.cycles = 1;

  rate2.count  = 1;
  rate2.cycles = 1;
}


/* ---------------------------------------------------------------- Checkers */


/*
 * Check for unique service name
 */
static void check_name(char *name) {

  ASSERT(name);

  if(Util_existService(name) || (current && IS(name, current->name))) {
    yyerror2("service name conflict, %s already defined", name);
  }
  if(name && *name=='/') {
    yyerror2("service name must not start with '/' -- ", name);
  }
  
}


/*
 * Permission statement semantic check
 */
static int check_perm(int perm) {

  long result;
  char *status;
  char buf[STRLEN];

  snprintf(buf, STRLEN, "%d", perm);

  result= strtol(buf, &status, 8);

  if( *status != '\0' || result < 0 || result > 07777 ) {
    
    yyerror2("permission statements must have an octal value "
             "between 0 and 7777");
        
  }

  return result;
  
}


/*
 * Timeout statement semantic check
 */
static void check_timeout(int s, int c) {

  if(s > c) {
    
    yyerror2("the number of restarts must be less than poll cycles");
    
  }

  if(s <= 0 || c <= 0) {
    
    yyerror2("zero or negative values not allowed in a timeout statement");
    
  }
  
}


/*
 * Every statement semantic check
 */
static void check_every(int every) {

  if(every <= 1) {
    
    yyerror2("an EVERY statement must have a value greater than 1");
    
  }
  
}


/*
 * Check hostname 
 */
static void check_hostname(char *hostname) {

  ASSERT(hostname);

  if(!check_host(hostname)) {
    
    yywarning2("hostname did not resolve");

  }
 
}

/*
 * Check the dependency graph for errors
 * by doing a topological sort, thereby finding any cycles.
 * Assures that graph is a Directed Acyclic Graph (DAG).
 */
static void check_depend() {
  Service_T s;
  Service_T depends_on= NULL;
  Service_T* dlt = &depend_list; /* the current tail of it                                 */
  int done;                      /* no unvisited nodes left?                               */
  int found_some;                /* last iteration found anything new ?                    */
  depend_list = NULL;            /* depend_list will be the topological sorted servicelist */

  do { 
    done = TRUE;
    found_some = FALSE; 
    for(s= servicelist; s; s= s->next) {
        Dependant_T d;
      if(s->visited)
	  continue;
      done = FALSE; // still unvisited nodes
      depends_on = NULL;
      for(d= s->dependantlist; d; d= d->next) {
        Service_T dp = Util_getService(d->dependant);
        if(!dp) {
          LogError("%s: Error: Depend service '%s' is not defined in the "
              "control file\n", prog, d->dependant);
          exit(1);
        }
        if (!dp->visited) {
          depends_on = dp;
        }
      }

      if (!depends_on) {
        s->visited = TRUE;
        found_some = TRUE;
        *dlt = s;
        dlt = &s->next_depend;
      }
    }
  } while(found_some && !done);
	
  if (!done)
   {
        ASSERT(depends_on);
	LogError("%s: Error: Found a depend loop in the control file "
	    "involving the service '%s'\n", prog, depends_on->name);
	exit(1);
   } 

  ASSERT(depend_list);
  servicelist= depend_list;
    
  for(s= depend_list; s; s= s->next_depend)
    s->next= s->next_depend;
    
  reset_depend();
}


/*
 * Check if the executable exist
 */
static void check_exec(char *exec) {
  
  if(! File_exist(exec)) {
    yyerror2("the executable does not exist");
  }
 
}
 

/* -------------------------------------------------------------------- Misc */


/*
 * Cleans up an md5 string, tolower and remove byte separators
 */
static int cleanup_hash_string(char *hashstring) {

  int i= 0, j= 0;

  ASSERT(hashstring);

  while (hashstring[i] != '\0') {

    if (isxdigit((int) hashstring[i])) {
      
      hashstring[j]=tolower((int)hashstring[i]);
      j++;

    } 

    i++;

  }
  
  hashstring[j]='\0';

  return j;
 
}


static void describeAction(Action_T A) {
  #define BUF_CURSOR    (A->description + strlen(A->description))
  #define BUF_AVAILABLE (sizeof(A->description) - strlen(A->description))
  snprintf(BUF_CURSOR, BUF_AVAILABLE, "%s", actionnames[A->id]);
  if(A->id == ACTION_EXEC) {
    int i = 0;
    Command_T C = A->exec;

    while(C->arg[i]) {
      snprintf(BUF_CURSOR, BUF_AVAILABLE, "%s%s", i?" ":" '", C->arg[i]);
      i++;
    }
    snprintf(BUF_CURSOR, BUF_AVAILABLE, "'");
    if(C->has_uid)
      snprintf(BUF_CURSOR, BUF_AVAILABLE, " as uid %d", C->uid);
    if(C->has_gid)
      snprintf(BUF_CURSOR, BUF_AVAILABLE, " as gid %d", C->gid);
    snprintf(BUF_CURSOR, BUF_AVAILABLE, " timeout %d cycle(s)", C->timeout);
  }
  #undef BUF_CURSOR
  #undef BUF_AVAILABLE
}


/* Return deep copy of the command */
static Command_T copycommand(Command_T source) {
  int i;
  Command_T copy = NULL;

  NEW(copy);
  copy->length = source->length;
  copy->has_uid = source->has_uid;
  copy->uid = source->uid;
  copy->has_gid = source->has_gid;
  copy->gid = source->gid;
  copy->timeout = source->timeout;
  for(i = 0; i < copy->length; i++) {
     copy->arg[i] = xstrdup(source->arg[i]);
  }
  copy->arg[copy->length]= NULL;

  return copy;
}

#line 3545 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = yyssp - yyss;
    newss = (yyss != 0)
          ? (short *)realloc(yyss, newsize * sizeof(*newss))
          : (short *)malloc(newsize * sizeof(*newss));
    if (newss == 0)
        return -1;

    yyss  = newss;
    yyssp = newss + i;
    newvs = (yyvs != 0)
          ? (YYSTYPE *)realloc(yyvs, newsize * sizeof(*newvs))
          : (YYSTYPE *)malloc(newsize * sizeof(*newvs));
    if (newvs == 0)
        return -1;

    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;

    yyerror("syntax error");

#ifdef lint
    goto yyerrlab;
#endif

yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 119:
#line 435 "p.y"
{
                    addmail(yyvsp[-5].string, &mailset, &Run.maillist, eventset, yyvsp[0].number);
                  }
break;
case 120:
#line 438 "p.y"
{
                    addmail(yyvsp[-2].string, &mailset, &Run.maillist, EVENT_ALL, yyvsp[0].number);
                  }
break;
case 121:
#line 441 "p.y"
{
                   addmail(yyvsp[-6].string, &mailset, &Run.maillist, ~eventset, yyvsp[0].number);
                  }
break;
case 122:
#line 446 "p.y"
{
                    if(!Run.isdaemon || ihp.daemon) {
                      ihp.daemon= TRUE;
                      Run.isdaemon= TRUE;
                      Run.polltime= yyvsp[0].number;
                    }
                  }
break;
case 123:
#line 455 "p.y"
{
                    Run.init= TRUE;
                  }
break;
case 124:
#line 460 "p.y"
{
                   if(!Run.logfile || ihp.logfile) {
                     ihp.logfile= TRUE;
                     setlogfile(yyvsp[0].string);
                     Run.use_syslog= FALSE;
                     Run.dolog=TRUE;
                   }
                  }
break;
case 125:
#line 468 "p.y"
{
                    setsyslog(NULL);
                  }
break;
case 126:
#line 471 "p.y"
{
                    setsyslog(yyvsp[0].string); FREE(yyvsp[0].string);
                  }
break;
case 127:
#line 476 "p.y"
{
                    Run.eventlist_dir= yyvsp[0].string;
                  }
break;
case 128:
#line 479 "p.y"
{
                    Run.eventlist_dir= yyvsp[-2].string;
                    Run.eventlist_slots= yyvsp[0].number;
                  }
break;
case 129:
#line 483 "p.y"
{
                    Run.eventlist_dir= xstrdup(MYEVENTLISTBASE);
                    Run.eventlist_slots= yyvsp[0].number;
                  }
break;
case 130:
#line 489 "p.y"
{
                    Run.statefile= yyvsp[0].string;
                  }
break;
case 131:
#line 494 "p.y"
{
                   if(!Run.pidfile || ihp.pidfile) {
                     ihp.pidfile= TRUE;
                     setpidfile(yyvsp[0].string);
                   }
                 }
break;
case 135:
#line 509 "p.y"
{
                    check_hostname((yyvsp[-3].url)->hostname);
                    addcollector(yyvsp[-3].url, yyvsp[-2].number, yyvsp[-1].number, yyvsp[0].string); 
                  }
break;
case 136:
#line 515 "p.y"
{
                   Run.mailserver_timeout= yyvsp[0].number;
                  }
break;
case 137:
#line 520 "p.y"
{
                   Run.MailFormat.from=
                     mailset.from?
                       mailset.from:
                       xstrdup(ALERT_FROM);
                   Run.MailFormat.subject=
                     mailset.subject?
                       mailset.subject:
                       xstrdup(ALERT_SUBJECT);
                   Run.MailFormat.message=
                     mailset.message?
                       mailset.message:
                       xstrdup(ALERT_MESSAGE);
                   reset_mailset();
                 }
break;
case 138:
#line 537 "p.y"
{
                   Run.dohttpd= TRUE;
                   Run.httpdport= yyvsp[-1].number;
                 }
break;
case 141:
#line 547 "p.y"
{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext= xstrdup(yyvsp[-4].string);

                    check_hostname(yyvsp[-4].string);
                    mailserverset.host = yyvsp[-4].string;
                    mailserverset.username = yyvsp[-3].string;
                    mailserverset.password = yyvsp[-2].string;
                    mailserverset.ssl.version= yyvsp[-1].number;
                    if(mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl= TRUE;
                      if(mailserverset.ssl.version == SSL_VERSION_SSLV2 ||
                         mailserverset.ssl.version == SSL_VERSION_SSLV3)
                         mailserverset.port = PORT_SMTPS;
                      mailserverset.ssl.certmd5= yyvsp[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 142:
#line 566 "p.y"
{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext= xstrdup(yyvsp[-6].string);

                    check_hostname(yyvsp[-6].string);
                    mailserverset.host = yyvsp[-6].string;
                    mailserverset.port = yyvsp[-4].number;
                    mailserverset.username = yyvsp[-3].string;
                    mailserverset.password = yyvsp[-2].string;
                    mailserverset.ssl.version= yyvsp[-1].number;
                    if(mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl= TRUE;
                      mailserverset.ssl.certmd5= yyvsp[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 149:
#line 595 "p.y"
{ Run.httpdssl= FALSE; }
break;
case 150:
#line 596 "p.y"
{ 
                    Run.httpdssl= TRUE;                   
                    if(!have_ssl()) {
                      yyerror("SSL is not supported");
                    }
                  }
break;
case 156:
#line 611 "p.y"
{ FREE(yyvsp[0].string); }
break;
case 157:
#line 612 "p.y"
{ FREE(yyvsp[0].string); }
break;
case 159:
#line 616 "p.y"
{ Run.httpdsig= TRUE; }
break;
case 160:
#line 617 "p.y"
{ Run.httpdsig= FALSE; }
break;
case 165:
#line 628 "p.y"
{ Run.bind_addr= yyvsp[0].string; }
break;
case 166:
#line 631 "p.y"
{
                    Run.httpsslpem= yyvsp[0].string;
                    if(!File_checkStat(Run.httpsslpem,
                          "SSL server PEM file", S_IRWXU)) {
                      yyerror2("SSL server PEM file has too loose permissions");
                    }
                  }
break;
case 168:
#line 641 "p.y"
{
                    Run.httpsslclientpem= yyvsp[0].string; 
                    Run.clientssl= TRUE;
                    if(!File_checkStat(Run.httpsslclientpem,
                          "SSL client PEM file", S_IRWXU | S_IRGRP | S_IROTH)) {
                      yyerror2("SSL client PEM file has too loose permissions");
                    }
                  }
break;
case 169:
#line 651 "p.y"
{ 
                   Run.allowselfcert= FALSE; 
                   config_ssl(Run.allowselfcert); 
                 }
break;
case 170:
#line 655 "p.y"
{   
                   Run.allowselfcert= TRUE;
                   config_ssl(Run.allowselfcert); 
                  }
break;
case 171:
#line 661 "p.y"
{
                    addcredentials(yyvsp[-3].string,yyvsp[-1].string, DIGEST_CLEARTEXT, yyvsp[0].number);
                  }
break;
case 172:
#line 664 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yyvsp[0].string);
                  }
break;
case 173:
#line 668 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yyvsp[0].string);
                  }
break;
case 174:
#line 672 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_MD5);
                    FREE(yyvsp[0].string);
                  }
break;
case 175:
#line 676 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CRYPT);
                    FREE(yyvsp[0].string);
                  }
break;
case 176:
#line 680 "p.y"
{
                    htpasswd_file= yyvsp[0].string;
                    digesttype= CLEARTEXT;
                  }
break;
case 177:
#line 684 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 178:
#line 687 "p.y"
{
                    htpasswd_file= yyvsp[0].string;
                    digesttype= DIGEST_CLEARTEXT;
                  }
break;
case 179:
#line 691 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 180:
#line 694 "p.y"
{
                    htpasswd_file= yyvsp[0].string;
                    digesttype= DIGEST_MD5;
                  }
break;
case 181:
#line 698 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 182:
#line 701 "p.y"
{
                    htpasswd_file= yyvsp[0].string;
                    digesttype= DIGEST_CRYPT;
                  }
break;
case 183:
#line 705 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 184:
#line 708 "p.y"
{
                    if(! (add_net_allow(yyvsp[0].string) || add_host_allow(yyvsp[0].string))) {
                      yyerror2("erroneous network or host identifier %s", yyvsp[0].string); 
                    }
                    FREE(yyvsp[0].string);
                  }
break;
case 187:
#line 720 "p.y"
{ addhtpasswdentry(htpasswd_file, yyvsp[0].string, digesttype);
                           FREE(yyvsp[0].string); }
break;
case 188:
#line 724 "p.y"
{ yyval.number= FALSE; }
break;
case 189:
#line 725 "p.y"
{ yyval.number= TRUE; }
break;
case 190:
#line 728 "p.y"
{
                   check_name(yyvsp[-2].string);
                   createservice(TYPE_PROCESS, yyvsp[-2].string, yyvsp[0].string, check_process);
                  }
break;
case 191:
#line 732 "p.y"
{
                   check_name(yyvsp[-2].string);
                   createservice(TYPE_PROCESS, yyvsp[-2].string, yyvsp[0].string, check_process);
                  }
break;
case 192:
#line 738 "p.y"
{
                   check_name(yyvsp[-2].string); 
                   createservice(TYPE_FILE, yyvsp[-2].string, yyvsp[0].string, check_file);
                  }
break;
case 193:
#line 744 "p.y"
{
                   check_name(yyvsp[-2].string);
                   createservice(TYPE_DEVICE, yyvsp[-2].string, yyvsp[0].string, check_device);
                  }
break;
case 194:
#line 750 "p.y"
{
                   check_name(yyvsp[-2].string);
                   createservice(TYPE_DIRECTORY, yyvsp[-2].string, yyvsp[0].string,
                                 check_directory);
                  }
break;
case 195:
#line 757 "p.y"
{
                   check_hostname(yyvsp[0].string); 
                   check_name(yyvsp[-2].string);
                   createservice(TYPE_HOST, yyvsp[-2].string, yyvsp[0].string, check_remote_host);
                  }
break;
case 196:
#line 764 "p.y"
{
                   check_name(yyvsp[0].string);
                   createservice(TYPE_SYSTEM, yyvsp[0].string, xstrdup(""),
                     check_system);
                   hassystem = TRUE;
                  }
break;
case 197:
#line 772 "p.y"
{
                   check_name(yyvsp[-2].string);
                   createservice(TYPE_FIFO, yyvsp[-2].string, yyvsp[0].string, check_fifo);
                  }
break;
case 198:
#line 778 "p.y"
{ addcommand(START); }
break;
case 199:
#line 779 "p.y"
{ addcommand(START); }
break;
case 200:
#line 782 "p.y"
{ addcommand(STOP); }
break;
case 201:
#line 783 "p.y"
{ addcommand(STOP); }
break;
case 206:
#line 794 "p.y"
{ addargument(yyvsp[0].string); }
break;
case 207:
#line 795 "p.y"
{ addargument(yyvsp[0].string); }
break;
case 208:
#line 798 "p.y"
{ addeuid( get_uid(yyvsp[0].string, 0) ); FREE(yyvsp[0].string); }
break;
case 209:
#line 799 "p.y"
{ addegid( get_gid(yyvsp[0].string, 0) ); FREE(yyvsp[0].string); }
break;
case 210:
#line 800 "p.y"
{ addeuid( get_uid(NULL, yyvsp[0].number) ); }
break;
case 211:
#line 801 "p.y"
{ addegid( get_gid(NULL, yyvsp[0].number) ); }
break;
case 212:
#line 804 "p.y"
{ yyval.string= NULL; }
break;
case 213:
#line 805 "p.y"
{ yyval.string= yyvsp[0].string; }
break;
case 214:
#line 808 "p.y"
{ yyval.string= NULL; }
break;
case 215:
#line 809 "p.y"
{ yyval.string= yyvsp[0].string; }
break;
case 216:
#line 813 "p.y"
{
                    portset.timeout= yyvsp[-4].number;
                    addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                    addport(&portset);
                  }
break;
case 217:
#line 819 "p.y"
{ 
                    prepare_urlrequest(yyvsp[-6].url);
                    portset.timeout= yyvsp[-4].number;
                    addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                    addport(&portset);
                  }
break;
case 218:
#line 828 "p.y"
{
                   portset.timeout= yyvsp[-4].number;
                   addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                   addport(&portset);
                  }
break;
case 219:
#line 836 "p.y"
{
                   icmpset.type= yyvsp[-6].number;
                   icmpset.count= yyvsp[-5].number;
                   icmpset.timeout= yyvsp[-4].number;
                   addeventaction(&(icmpset).action, yyvsp[-1].number, yyvsp[0].number);
                   addicmp(&icmpset);
                  }
break;
case 220:
#line 845 "p.y"
{
                    if(current->type == TYPE_HOST)
                      portset.hostname= xstrdup(current->path);
                    else
                      portset.hostname= xstrdup(LOCALHOST);
                  }
break;
case 221:
#line 851 "p.y"
{ check_hostname(yyvsp[0].string); portset.hostname= yyvsp[0].string; }
break;
case 222:
#line 854 "p.y"
{ portset.port= yyvsp[0].number; portset.family= AF_INET; }
break;
case 223:
#line 857 "p.y"
{
                    portset.pathname= yyvsp[0].string; portset.family= AF_UNIX;
                  }
break;
case 224:
#line 862 "p.y"
{
                    portset.type= SOCK_STREAM;
                  }
break;
case 225:
#line 865 "p.y"
{
                    portset.type= SOCK_STREAM;
                  }
break;
case 226:
#line 868 "p.y"
{
                    portset.type=       SOCK_STREAM;
                    portset.SSL.use_ssl= TRUE;
                    portset.SSL.version= yyvsp[-1].number;
                    if(portset.SSL.version == SSL_VERSION_NONE)
                      portset.SSL.version = SSL_VERSION_AUTO;
                    portset.SSL.certmd5= yyvsp[0].string;
                  }
break;
case 227:
#line 876 "p.y"
{
                    portset.type= SOCK_DGRAM;
                  }
break;
case 228:
#line 881 "p.y"
{ yyval.string= NULL; }
break;
case 229:
#line 882 "p.y"
{ yyval.string= yyvsp[0].string; }
break;
case 230:
#line 885 "p.y"
{ yyval.number= SSL_VERSION_NONE; }
break;
case 231:
#line 886 "p.y"
{ yyval.number= SSL_VERSION_SSLV2; }
break;
case 232:
#line 887 "p.y"
{ yyval.number= SSL_VERSION_SSLV3; }
break;
case 233:
#line 888 "p.y"
{ yyval.number= SSL_VERSION_TLS; }
break;
case 234:
#line 889 "p.y"
{ yyval.number= SSL_VERSION_AUTO; }
break;
case 235:
#line 892 "p.y"
{
                    portset.protocol= addprotocol(P_DEFAULT);
                  }
break;
case 236:
#line 895 "p.y"
{
                    portset.protocol= addprotocol(P_APACHESTATUS);
                  }
break;
case 237:
#line 898 "p.y"
{
                    portset.protocol= addprotocol(P_DEFAULT);
                  }
break;
case 238:
#line 901 "p.y"
{
                    portset.protocol= addprotocol(P_DNS);
                  }
break;
case 239:
#line 904 "p.y"
{
                    portset.protocol= addprotocol(P_DWP);
                  }
break;
case 240:
#line 907 "p.y"
{
                    portset.protocol= addprotocol(P_FTP);
                  }
break;
case 241:
#line 910 "p.y"
{
                    portset.protocol= addprotocol(P_HTTP);
                  }
break;
case 242:
#line 913 "p.y"
{
                    portset.protocol= addprotocol(P_IMAP);
                  }
break;
case 243:
#line 916 "p.y"
{
                    portset.protocol= addprotocol(P_CLAMAV);
                  }
break;
case 244:
#line 919 "p.y"
{
                    portset.protocol= addprotocol(P_LDAP2);
                  }
break;
case 245:
#line 922 "p.y"
{
                    portset.protocol= addprotocol(P_LDAP3);
                  }
break;
case 246:
#line 925 "p.y"
{
                    portset.protocol= addprotocol(P_MYSQL);
                  }
break;
case 247:
#line 928 "p.y"
{
                    portset.protocol= addprotocol(P_NNTP);
                  }
break;
case 248:
#line 931 "p.y"
{
                    portset.protocol= addprotocol(P_NTP3);
                    portset.type= SOCK_DGRAM;
                  }
break;
case 249:
#line 935 "p.y"
{
                    portset.protocol= addprotocol(P_POSTFIXPOLICY);
                  }
break;
case 250:
#line 938 "p.y"
{
                    portset.protocol= addprotocol(P_POP);
                  }
break;
case 251:
#line 941 "p.y"
{
                    portset.protocol= addprotocol(P_SMTP);
                  }
break;
case 252:
#line 944 "p.y"
{
                    portset.protocol= addprotocol(P_SSH);
                  }
break;
case 253:
#line 947 "p.y"
{
                    portset.protocol= addprotocol(P_RDATE);
                  }
break;
case 254:
#line 950 "p.y"
{
                    portset.protocol= addprotocol(P_RSYNC);
                  }
break;
case 255:
#line 953 "p.y"
{
                    portset.protocol= addprotocol(P_TNS);
                  }
break;
case 256:
#line 956 "p.y"
{
                    portset.protocol= addprotocol(P_PGSQL);
                  }
break;
case 257:
#line 959 "p.y"
{
                    portset.protocol= addprotocol(P_GENERIC);
                  }
break;
case 260:
#line 968 "p.y"
{ addgeneric(&portset, yyvsp[0].string, NULL); FREE(yyvsp[0].string);}
break;
case 261:
#line 969 "p.y"
{ addgeneric(&portset, NULL, yyvsp[0].string); FREE(yyvsp[0].string);}
break;
case 263:
#line 973 "p.y"
{ 
                    portset.request= Util_urlEncode(yyvsp[0].string); 
                    FREE(yyvsp[0].string); 
                  }
break;
case 264:
#line 977 "p.y"
{
                    portset.request= Util_urlEncode(yyvsp[-2].string); FREE(yyvsp[-2].string);
                    portset.request_checksum= yyvsp[0].string;
                  }
break;
case 267:
#line 987 "p.y"
{ 
                    portset.ApacheStatus.loglimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.loglimit= (int)yyvsp[-1].number; 
                  }
break;
case 268:
#line 991 "p.y"
{ 
                    portset.ApacheStatus.closelimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.closelimit= (int)(yyvsp[-1].number); 
                  }
break;
case 269:
#line 995 "p.y"
{ 
                    portset.ApacheStatus.dnslimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.dnslimit= (int)(yyvsp[-1].number); 
                  }
break;
case 270:
#line 999 "p.y"
{ 
                    portset.ApacheStatus.keepalivelimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.keepalivelimit= (int)(yyvsp[-1].number); 
                  }
break;
case 271:
#line 1003 "p.y"
{ 
                    portset.ApacheStatus.replylimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.replylimit= (int)(yyvsp[-1].number); 
                  }
break;
case 272:
#line 1007 "p.y"
{ 
                    portset.ApacheStatus.requestlimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.requestlimit= (int)(yyvsp[-1].number); 
                  }
break;
case 273:
#line 1011 "p.y"
{ 
                    portset.ApacheStatus.startlimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.startlimit= (int)(yyvsp[-1].number); 
                  }
break;
case 274:
#line 1015 "p.y"
{ 
                    portset.ApacheStatus.waitlimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.waitlimit= (int)(yyvsp[-1].number); 
                  }
break;
case 275:
#line 1019 "p.y"
{ 
                    portset.ApacheStatus.gracefullimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.gracefullimit= (int)(yyvsp[-1].number); 
                  }
break;
case 276:
#line 1023 "p.y"
{ 
                    portset.ApacheStatus.cleanuplimitOP= yyvsp[-2].number; 
                    portset.ApacheStatus.cleanuplimit= (int)(yyvsp[-1].number); 
                  }
break;
case 277:
#line 1029 "p.y"
{
                    seteventaction(&(current)->action_PID, yyvsp[0].number,
                      ACTION_IGNORE);
                  }
break;
case 278:
#line 1035 "p.y"
{
                    seteventaction(&(current)->action_PPID, yyvsp[0].number,
                      ACTION_IGNORE);
                  }
break;
case 279:
#line 1041 "p.y"
{
                   yyval.number= ICMP_ATTEMPT_COUNT;
                  }
break;
case 280:
#line 1044 "p.y"
{    
                   yyval.number= yyvsp[0].number;
                  }
break;
case 281:
#line 1049 "p.y"
{
                   yyval.number= NET_TIMEOUT;
                  }
break;
case 282:
#line 1052 "p.y"
{
                   yyval.number= yyvsp[-1].number;
                  }
break;
case 283:
#line 1057 "p.y"
{
                   check_timeout(yyvsp[-5].number, yyvsp[-3].number);
                   current->def_timeout= TRUE;
                   current->to_start= yyvsp[-5].number;
                   current->to_cycle= yyvsp[-3].number;
                 }
break;
case 285:
#line 1066 "p.y"
{
                    seturlrequest(yyvsp[-1].number, yyvsp[0].string);
                    FREE(yyvsp[0].string);
                  }
break;
case 286:
#line 1072 "p.y"
{ yyval.number= OPERATOR_EQUAL; }
break;
case 287:
#line 1073 "p.y"
{ yyval.number= OPERATOR_NOTEQUAL; }
break;
case 288:
#line 1076 "p.y"
{
                   addmail(yyvsp[-5].string, &mailset, &current->maillist, eventset, yyvsp[0].number);
                  }
break;
case 289:
#line 1079 "p.y"
{
                   addmail(yyvsp[-2].string, &mailset, &current->maillist, EVENT_ALL, yyvsp[0].number);
                  }
break;
case 290:
#line 1082 "p.y"
{
                   addmail(yyvsp[-6].string, &mailset, &current->maillist, ~eventset, yyvsp[0].number);
                  }
break;
case 291:
#line 1085 "p.y"
{
                   addmail(yyvsp[0].string, &mailset, &current->maillist, EVENT_NULL, 0);
                  }
break;
case 292:
#line 1090 "p.y"
{ yyval.string= yyvsp[0].string; }
break;
case 293:
#line 1093 "p.y"
{ yyval.string= yyvsp[0].string; }
break;
case 296:
#line 1100 "p.y"
{ eventset |= EVENT_CHANGED; }
break;
case 297:
#line 1101 "p.y"
{ eventset |= EVENT_CHECKSUM; }
break;
case 298:
#line 1102 "p.y"
{ eventset |= EVENT_CONNECTION; }
break;
case 299:
#line 1103 "p.y"
{ eventset |= EVENT_DATA; }
break;
case 300:
#line 1104 "p.y"
{ eventset |= EVENT_EXEC; }
break;
case 301:
#line 1105 "p.y"
{ eventset |= EVENT_GID; }
break;
case 302:
#line 1106 "p.y"
{ eventset |= EVENT_ICMP; }
break;
case 303:
#line 1107 "p.y"
{ eventset |= EVENT_INSTANCE; }
break;
case 304:
#line 1108 "p.y"
{ eventset |= EVENT_INVALID; }
break;
case 305:
#line 1109 "p.y"
{ eventset |= EVENT_MATCH; }
break;
case 306:
#line 1110 "p.y"
{ eventset |= EVENT_NONEXIST; }
break;
case 307:
#line 1111 "p.y"
{ eventset |= EVENT_PERMISSION; }
break;
case 308:
#line 1112 "p.y"
{ eventset |= EVENT_RESOURCE; }
break;
case 309:
#line 1113 "p.y"
{ eventset |= EVENT_SIZE; }
break;
case 310:
#line 1114 "p.y"
{ eventset |= EVENT_TIMEOUT; }
break;
case 311:
#line 1115 "p.y"
{ eventset |= EVENT_TIMESTAMP; }
break;
case 312:
#line 1116 "p.y"
{ eventset |= EVENT_UID; }
break;
case 313:
#line 1119 "p.y"
{ eventset |= EVENT_NONEXIST|EVENT_INVALID; }
break;
case 314:
#line 1120 "p.y"
{ eventset |= EVENT_NONEXIST|EVENT_INVALID; }
break;
case 315:
#line 1121 "p.y"
{ eventset |= EVENT_NONEXIST|EVENT_INVALID; }
break;
case 316:
#line 1122 "p.y"
{ eventset |= EVENT_TIMEOUT; }
break;
case 321:
#line 1133 "p.y"
{ mailset.from= yyvsp[0].string; }
break;
case 322:
#line 1134 "p.y"
{ mailset.subject= yyvsp[0].string; }
break;
case 323:
#line 1135 "p.y"
{ mailset.message= yyvsp[0].string; }
break;
case 324:
#line 1138 "p.y"
{
                   check_every(yyvsp[-1].number);
                   current->def_every= TRUE;
                   current->every= yyvsp[-1].number;
                 }
break;
case 325:
#line 1145 "p.y"
{
                    current->mode= MODE_ACTIVE;
                  }
break;
case 326:
#line 1148 "p.y"
{
                    current->mode= MODE_PASSIVE;
                  }
break;
case 327:
#line 1151 "p.y"
{
                    current->mode= MODE_MANUAL;
                    current->monitor= MONITOR_NOT;
                  }
break;
case 328:
#line 1157 "p.y"
{ current->group= yyvsp[0].string; }
break;
case 332:
#line 1168 "p.y"
{ adddependant(yyvsp[0].string); }
break;
case 333:
#line 1171 "p.y"
{
                     addeventaction(&(resourceset).action,
                       yyvsp[-1].number, yyvsp[0].number);
                     addresource(&resourceset);
                   }
break;
case 340:
#line 1188 "p.y"
{
                     addeventaction(&(resourceset).action,
                       yyvsp[-1].number, yyvsp[0].number);
                     addresource(&resourceset);
                   }
break;
case 346:
#line 1204 "p.y"
{
                    resourceset.resource_id= RESOURCE_ID_CPU_PERCENT;
                    resourceset.operator= yyvsp[-2].number;
                    resourceset.limit= (yyvsp[-1].number * 10); 
                  }
break;
case 347:
#line 1211 "p.y"
{
                    resourceset.resource_id= yyvsp[-3].number;
                    resourceset.operator= yyvsp[-2].number;
                    resourceset.limit= (yyvsp[-1].number * 10); 
                  }
break;
case 348:
#line 1218 "p.y"
{ yyval.number= RESOURCE_ID_CPUUSER; }
break;
case 349:
#line 1219 "p.y"
{ yyval.number= RESOURCE_ID_CPUSYSTEM; }
break;
case 350:
#line 1220 "p.y"
{ yyval.number= RESOURCE_ID_CPUWAIT; }
break;
case 351:
#line 1223 "p.y"
{
                    resourceset.resource_id= RESOURCE_ID_MEM_KBYTE;
                    resourceset.operator= yyvsp[-2].number;
                    resourceset.limit= (int) (yyvsp[-1].real * 
                      (yyvsp[0].number / 1024.0)); 
                  }
break;
case 352:
#line 1229 "p.y"
{
                    resourceset.resource_id= RESOURCE_ID_MEM_PERCENT;
                    resourceset.operator= yyvsp[-2].number;
                    resourceset.limit= (yyvsp[-1].number * 10); 
                  }
break;
case 353:
#line 1234 "p.y"
{
                    resourceset.resource_id= RESOURCE_ID_TOTAL_MEM_KBYTE;
                    resourceset.operator= yyvsp[-2].number;
                    resourceset.limit= (int) (yyvsp[-1].real * 
                      (yyvsp[0].number / 1024.0));
                  }
break;
case 354:
#line 1240 "p.y"
{
                    resourceset.resource_id= RESOURCE_ID_TOTAL_MEM_PERCENT;
                    resourceset.operator= yyvsp[-2].number;
                    resourceset.limit= (yyvsp[-1].number * 10);
                  }
break;
case 355:
#line 1247 "p.y"
{ 
                    resourceset.resource_id= RESOURCE_ID_CHILDREN;
                    resourceset.operator= yyvsp[-1].number;
                    resourceset.limit= (int) yyvsp[0].number; 
                  }
break;
case 356:
#line 1254 "p.y"
{ 
                    resourceset.resource_id= yyvsp[-2].number;
                    resourceset.operator= yyvsp[-1].number;
                    resourceset.limit= (int) (yyvsp[0].real * 10.0); 
                  }
break;
case 357:
#line 1261 "p.y"
{ yyval.number= RESOURCE_ID_LOAD1; }
break;
case 358:
#line 1262 "p.y"
{ yyval.number= RESOURCE_ID_LOAD5; }
break;
case 359:
#line 1263 "p.y"
{ yyval.number= RESOURCE_ID_LOAD15; }
break;
case 360:
#line 1266 "p.y"
{ yyval.real = yyvsp[0].real; }
break;
case 361:
#line 1267 "p.y"
{ yyval.real = (float) yyvsp[0].number; }
break;
case 362:
#line 1271 "p.y"
{
                    timestampset.operator= yyvsp[-6].number;
                    timestampset.time= (yyvsp[-5].number * yyvsp[-4].number);
                    addeventaction(&(timestampset).action, yyvsp[-1].number,
                      yyvsp[0].number);
                    addtimestamp(&timestampset, FALSE);
                  }
break;
case 363:
#line 1278 "p.y"
{
                    timestampset.test_changes= TRUE;
                    addeventaction(&(timestampset).action, yyvsp[0].number,
                      ACTION_IGNORE);
                    addtimestamp(&timestampset, TRUE);
                  }
break;
case 364:
#line 1286 "p.y"
{ yyval.number= OPERATOR_EQUAL; }
break;
case 365:
#line 1287 "p.y"
{ yyval.number= OPERATOR_GREATER; }
break;
case 366:
#line 1288 "p.y"
{ yyval.number= OPERATOR_LESS; }
break;
case 367:
#line 1289 "p.y"
{ yyval.number= OPERATOR_EQUAL; }
break;
case 368:
#line 1290 "p.y"
{ yyval.number= OPERATOR_NOTEQUAL; }
break;
case 369:
#line 1291 "p.y"
{ yyval.number= OPERATOR_NOTEQUAL; }
break;
case 370:
#line 1294 "p.y"
{ yyval.number= TIME_SECOND; }
break;
case 371:
#line 1295 "p.y"
{ yyval.number= TIME_SECOND; }
break;
case 372:
#line 1296 "p.y"
{ yyval.number= TIME_MINUTE; }
break;
case 373:
#line 1297 "p.y"
{ yyval.number= TIME_HOUR; }
break;
case 374:
#line 1298 "p.y"
{ yyval.number= TIME_DAY; }
break;
case 375:
#line 1301 "p.y"
{ yyval.number= ACTION_ALERT; }
break;
case 376:
#line 1302 "p.y"
{ yyval.number= ACTION_EXEC; }
break;
case 377:
#line 1303 "p.y"
{ yyval.number= ACTION_EXEC; }
break;
case 378:
#line 1304 "p.y"
{ yyval.number= ACTION_RESTART; }
break;
case 379:
#line 1305 "p.y"
{ yyval.number= ACTION_START; }
break;
case 380:
#line 1306 "p.y"
{ yyval.number= ACTION_STOP; }
break;
case 381:
#line 1307 "p.y"
{ yyval.number= ACTION_UNMONITOR; }
break;
case 382:
#line 1310 "p.y"
{
                    yyval.number= yyvsp[0].number;
                    if(yyvsp[0].number == ACTION_EXEC && command) {
                      command1= command;
                      command= NULL;
                    }
                  }
break;
case 383:
#line 1319 "p.y"
{
                    yyval.number= yyvsp[0].number;
                    if(yyvsp[0].number == ACTION_EXEC && command) {
                      command2= command;
                      command= NULL;
                    }
                  }
break;
case 385:
#line 1329 "p.y"
{
                    rate1.count  = yyvsp[-1].number;
                    rate1.cycles = yyvsp[-1].number;
                    if(rate1.cycles < 1 || rate1.cycles > BITMAP_MAX) {
                      yyerror2("the number of cycles must be between 1 and %d",
                               BITMAP_MAX);
                    }
                  }
break;
case 386:
#line 1337 "p.y"
{
                    rate1.count  = yyvsp[-2].number;
                    rate1.cycles = yyvsp[-1].number;
                    if(rate1.cycles < 1 || rate1.cycles > BITMAP_MAX) {
                      yyerror2("the number of cycles must be between 1 and %d",
                               BITMAP_MAX);
                    }
                    if(rate1.count < 1 || rate1.count > rate1.cycles) {
                      yyerror2("the number of events must be bigger then 0 and "
                               "less than poll cycles");
                    }
                  }
break;
case 388:
#line 1352 "p.y"
{
                    rate2.count  = yyvsp[-1].number;
                    rate2.cycles = yyvsp[-1].number;
                    if(rate2.cycles < 1 || rate2.cycles > BITMAP_MAX) {
                      yyerror2("the number of cycles must be between 1 and %d",
                               BITMAP_MAX);
                    }
                  }
break;
case 389:
#line 1360 "p.y"
{
                    rate2.count  = yyvsp[-2].number;
                    rate2.cycles = yyvsp[-1].number;
                    if(rate2.cycles < 1 || rate2.cycles > BITMAP_MAX) {
                      yyerror2("the number of cycles must be between 1 and %d",
                               BITMAP_MAX);
                    }
                    if(rate2.count < 1 || rate2.count > rate2.cycles) {
                      yyerror2("the number of events must be bigger then 0 and "
                               "less than poll cycles");
                    }
                  }
break;
case 390:
#line 1374 "p.y"
{
                    yyval.number= ACTION_ALERT;
                  }
break;
case 391:
#line 1377 "p.y"
{
                    yyval.number= yyvsp[0].number;
                  }
break;
case 392:
#line 1380 "p.y"
{
                    yyval.number= yyvsp[0].number;
                  }
break;
case 393:
#line 1385 "p.y"
{
                    addeventaction(&(checksumset).action, yyvsp[-1].number,
                      yyvsp[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 394:
#line 1391 "p.y"
{
                    checksumset.hash= yyvsp[-4].string;
                    addeventaction(&(checksumset).action, yyvsp[-1].number,
                      yyvsp[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 395:
#line 1397 "p.y"
{
                    checksumset.test_changes= TRUE;
                    addeventaction(&(checksumset).action, yyvsp[0].number,
                      ACTION_IGNORE);
                    addchecksum(&checksumset);
                  }
break;
case 396:
#line 1404 "p.y"
{ checksumset.type= HASH_UNKNOWN; }
break;
case 397:
#line 1405 "p.y"
{ checksumset.type= HASH_MD5; }
break;
case 398:
#line 1406 "p.y"
{ checksumset.type= HASH_SHA1; }
break;
case 399:
#line 1409 "p.y"
{
                    deviceset.resource= RESOURCE_ID_INODE;
                    deviceset.operator= yyvsp[-5].number;
                    deviceset.limit_absolute= yyvsp[-4].number;
                    addeventaction(&(deviceset).action, yyvsp[-1].number, yyvsp[0].number);
                    adddevice(&deviceset);
                  }
break;
case 400:
#line 1416 "p.y"
{
                    deviceset.resource= RESOURCE_ID_INODE;
                    deviceset.operator= yyvsp[-6].number;
                    deviceset.limit_percent= (int)(yyvsp[-5].number * 10);
                    addeventaction(&(deviceset).action, yyvsp[-1].number, yyvsp[0].number);
                    adddevice(&deviceset);
                  }
break;
case 401:
#line 1425 "p.y"
{
                    if(!device_usage(current->inf, current->path)) {
                      yyerror2("cannot read usage of device %s",
                               current->path);
                    }
                    deviceset.resource= RESOURCE_ID_SPACE;
                    deviceset.operator= yyvsp[-6].number;
                    deviceset.limit_absolute=
                      (int)(
                        (float)yyvsp[-5].real /
                        (float)current->inf->f_bsize * (float)yyvsp[-4].number
                      );
                    addeventaction(&(deviceset).action, yyvsp[-1].number, yyvsp[0].number);
                    adddevice(&deviceset);
                  }
break;
case 402:
#line 1440 "p.y"
{
                    deviceset.resource= RESOURCE_ID_SPACE;
                    deviceset.operator= yyvsp[-6].number;
                    deviceset.limit_percent= (int)(yyvsp[-5].number * 10);
                    addeventaction(&(deviceset).action, yyvsp[-1].number, yyvsp[0].number);
                    adddevice(&deviceset);
                  }
break;
case 403:
#line 1449 "p.y"
{
                    seteventaction(&(current)->action_FSFLAG, yyvsp[0].number,
                      ACTION_IGNORE);
                  }
break;
case 404:
#line 1455 "p.y"
{ yyval.number= UNIT_BYTE; }
break;
case 405:
#line 1456 "p.y"
{ yyval.number= UNIT_KILOBYTE; }
break;
case 406:
#line 1457 "p.y"
{ yyval.number= UNIT_MEGABYTE; }
break;
case 407:
#line 1458 "p.y"
{ yyval.number= UNIT_GIGABYTE; }
break;
case 408:
#line 1461 "p.y"
{
                    permset.perm= check_perm(yyvsp[-4].number);
                    addeventaction(&(permset).action, yyvsp[-1].number, yyvsp[0].number);
                    addperm(&permset);
                  }
break;
case 409:
#line 1468 "p.y"
{
                    matchset.ignore= FALSE;
                    matchset.match_path= yyvsp[-3].string;
                    matchset.match_string= NULL;
                    addmatchpath(&matchset, yyvsp[0].number);
                    FREE(yyvsp[-3].string); 
                  }
break;
case 410:
#line 1475 "p.y"
{
                    matchset.ignore= FALSE;
                    matchset.match_path= NULL;
                    matchset.match_string= xstrdup(yyvsp[-3].string);
                    addmatch(&matchset, yyvsp[0].number, 0);
                  }
break;
case 411:
#line 1481 "p.y"
{
                    matchset.ignore= TRUE;
                    matchset.match_path= yyvsp[0].string;
                    matchset.match_string= NULL;
                    addmatchpath(&matchset, ACTION_IGNORE);
                    FREE(yyvsp[0].string); 
                  }
break;
case 412:
#line 1488 "p.y"
{
                    matchset.ignore= TRUE;
                    matchset.match_path= NULL;
                    matchset.match_string= xstrdup(yyvsp[0].string);
                    addmatch(&matchset, ACTION_IGNORE, 0);
                  }
break;
case 413:
#line 1496 "p.y"
{
                    matchset.not= FALSE;
                  }
break;
case 414:
#line 1499 "p.y"
{
                    matchset.not= TRUE;
                  }
break;
case 415:
#line 1505 "p.y"
{
                    sizeset.operator= yyvsp[-6].number;
                    sizeset.size= ((unsigned long long)yyvsp[-5].number * yyvsp[-4].number);
                    addeventaction(&(sizeset).action, yyvsp[-1].number, yyvsp[0].number);
                    addsize(&sizeset, FALSE);
                  }
break;
case 416:
#line 1511 "p.y"
{
                    sizeset.test_changes= TRUE;
                    addeventaction(&(sizeset).action, yyvsp[0].number, 
                      ACTION_IGNORE);
                    addsize(&sizeset, TRUE);
                  }
break;
case 417:
#line 1519 "p.y"
{
                    uidset.uid= get_uid(yyvsp[-4].string, 0);
                    addeventaction(&(uidset).action, yyvsp[-1].number, yyvsp[0].number);
                    adduid(&uidset);
                    FREE(yyvsp[-4].string);
                  }
break;
case 418:
#line 1525 "p.y"
{
                    uidset.uid= get_uid(NULL, yyvsp[-4].number);
                    addeventaction(&(uidset).action, yyvsp[-1].number, yyvsp[0].number);
                    adduid(&uidset);
                  }
break;
case 419:
#line 1532 "p.y"
{
                    gidset.gid= get_gid(yyvsp[-4].string, 0);
                    addeventaction(&(gidset).action, yyvsp[-1].number, yyvsp[0].number);
                    addgid(&gidset);
                    FREE(yyvsp[-4].string);
                  }
break;
case 420:
#line 1538 "p.y"
{
                    gidset.gid= get_gid(NULL, yyvsp[-4].number);
                    addeventaction(&(gidset).action, yyvsp[-1].number, yyvsp[0].number);
                    addgid(&gidset);
                  }
break;
case 421:
#line 1545 "p.y"
{ yyval.number= ICMP_ECHO; }
break;
case 422:
#line 1548 "p.y"
{ yyval.number = 0; }
break;
case 423:
#line 1549 "p.y"
{ yyval.number = yyvsp[0].number; }
break;
case 424:
#line 1550 "p.y"
{ yyval.number = yyvsp[-1].number; }
break;
#line 5291 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;

yyoverflow:
    yyerror("yacc stack overflow");

yyabort:
    return (1);

yyaccept:
    return (0);
}
